@@ -1,9 +1,10 @@
-/* Time-stamp: <2005-03-02 18:36:05 jcs>
+/* Time-stamp: <2005-06-17 22:25:31 jcs>
 |
-|  Copyright (C) 2002-2003 Jorg Schuler <jcsjcs at users.sourceforge.net>
+|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
-|  URL:  http://gtkpod.sourceforge.net/
+|  URL: http://www.gtkpod.org/
+|  URL: http://gtkpod.sourceforge.net/
 | 
 |  This program is free software; you can redistribute it and/or modify
 |  it under the terms of the GNU General Public License as published by
@@ -23,7 +24,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: charset.c,v 1.19 2004/12/16 13:52:50 jcsjcs Exp $
+|  $Id: charset.c,v 1.23 2005/06/17 13:30:02 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -35,7 +36,6 @@
 #include "prefs.h"
 #include "charset.h"
 #include "misc.h"
-#include "support.h"
 
 /* If Japanese auto-conversion is being used, this variable is being
    set with each call of charset_to_utf8(). You can get a copy of its
@@ -371,11 +371,18 @@ gchar *charset_from_utf8 (const gchar *str)
 /* Must free the returned string yourself */
 gchar *charset_track_charset_from_utf8 (Track *s, const gchar *str)
 {
-    const gchar *charset;
+    const gchar *charset = NULL;
+    ExtraTrackData *etd;
+
+    g_return_val_if_fail (s, NULL);
+    g_return_val_if_fail (s->userdata, NULL);
 
     if (str == NULL) return NULL;  /* sanity */
-    if (s && s->charset && strlen (s->charset))
-	   charset = s->charset;
+
+    etd = s->userdata;
+
+    if (etd->charset && strlen (etd->charset))
+	   charset = etd->charset;
     else   charset = prefs_get_charset ();
     if (!charset || !strlen (charset))
     {    /* use standard locale charset */
@@ -456,8 +463,8 @@ gchar *charset_to_charset (const gchar *from_charset,
 	    }
 	    if (ret == NULL)
 	    { /* Well... what do you think we should return? */
-		ret = g_strdup (_("gtkpod: Invalid Conversion"));
-		/* ret = g_strdup ("");*/
+		/* ret = g_strdup (_("gtkpod: Invalid Conversion")); */
+		ret = g_strdup ("");
 	    }
 	}
 	g_free (strc);
