@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-06-25 00:35:45 jcs>
+/* Time-stamp: <2005-12-10 14:43:34 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display_playlists.c,v 1.60 2005/06/24 16:17:18 jcsjcs Exp $
+|  $Id: display_playlists.c,v 1.71 2005/12/10 08:21:59 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -66,6 +66,10 @@ static GtkTargetEntry pm_drop_types [] = {
 };
 
 
+/* SEMIBOLD was introduced with version 1.8 */
+#ifndef PANGO_WEIGHT_SEMIBOLD
+#define PANGO_WEIGHT_SEMIBOLD 600
+#endif
 
 
 /* ---------------------------------------------------------------- */
@@ -437,12 +441,12 @@ static GdkDragAction pm_pm_get_action (Playlist *src, Playlist *dest,
 	NULL, NULL, &mask);
 
     /* don't allow copy/move before the MPL */
-    if ((dest->type == ITDB_PL_TYPE_MPL) &&
+    if ((itdb_playlist_is_mpl (dest)) && 
 	(pos == GTK_TREE_VIEW_DROP_BEFORE))
 	return 0;
 
     /* don't allow moving of MPL */
-    if (src->type == ITDB_PL_TYPE_MPL)
+    if (itdb_playlist_is_mpl (src))
 	return GDK_ACTION_COPY;
 
     /* don't allow drop onto itself */
@@ -454,7 +458,7 @@ static GdkDragAction pm_pm_get_action (Playlist *src, Playlist *dest,
     if (src->itdb == dest->itdb)
     {   /* DND within the same itdb */
         /* don't allow copy/move onto MPL */
-	if ((dest->type == ITDB_PL_TYPE_MPL) &&
+	if ((itdb_playlist_is_mpl (dest)) &&
 	    (pos != GTK_TREE_VIEW_DROP_AFTER))
 	    return 0;
 
@@ -515,14 +519,14 @@ static GdkDragAction pm_tm_get_action (Track *src, Playlist *dest,
 
 
     /* don't allow copy/move before the MPL */
-    if ((dest->type == ITDB_PL_TYPE_MPL) &&
+    if ((itdb_playlist_is_mpl (dest)) &&
 	(pos == GTK_TREE_VIEW_DROP_BEFORE))
 	return 0;
 
     if (src->itdb == dest->itdb)
     {   /* DND within the same itdb */
         /* don't allow copy/move onto MPL */
-	if ((dest->type == ITDB_PL_TYPE_MPL) &&
+	if ((itdb_playlist_is_mpl (dest)) &&
 	    (pos != GTK_TREE_VIEW_DROP_AFTER))
 	    return 0;
     }
@@ -607,7 +611,7 @@ static void pm_drag_data_received (GtkWidget       *widget,
 
     if (path_m)
     {   /* this callback was caused by pm_drag_motion -- we are
-	 * supposed to gdk_drag_status () */
+	 * supposed to call gdk_drag_status () */
 /* puts ("...by motion"); */
         pos = (GtkTreeViewDropPosition)g_object_get_data (G_OBJECT (widget), "drag_data_by_motion_pos");
         /* unset flag that */ 
@@ -999,7 +1003,7 @@ void pm_remove_track (Playlist *playlist, Track *track)
 	if (track->itdb == current_playlist->itdb)
 	{
 	    if ((playlist == current_playlist) ||
-		(current_playlist->type == ITDB_PL_TYPE_MPL))
+		itdb_playlist_is_mpl (current_playlist))
 	    {
 		st_remove_track (track, 0);
 	    }
@@ -1079,7 +1083,7 @@ void pm_add_playlist (Playlist *playlist, gint pos)
   model = GTK_TREE_MODEL (gtk_tree_view_get_model (playlist_treeview));
   g_return_if_fail (model);
 
-  if (playlist->type == ITDB_PL_TYPE_MPL)
+  if (itdb_playlist_is_mpl (playlist))
   {   /* MPLs are always added top-level */
       mpl = NULL;
   }
@@ -1127,7 +1131,7 @@ void pm_add_playlist (Playlist *playlist, gint pos)
       /* If it's the first playlist (no playlist selected AND playlist is
 	 the MPL, we select it, unless prefs_get_mpl_autoselect() says
 	 otherwise */
-      if ((playlist->type == ITDB_PL_TYPE_MPL) && prefs_get_mpl_autoselect())
+      if (itdb_playlist_is_mpl(playlist) && prefs_get_mpl_autoselect())
       {
 	  selection = gtk_tree_view_get_selection (playlist_treeview);
 	  gtk_tree_selection_select_iter (selection, &iter);
@@ -1143,9 +1147,9 @@ void pm_add_playlist (Playlist *playlist, gint pos)
 void pm_remove_playlist (Playlist *playlist, gboolean select)
 {
     gboolean pm_delete_playlist_fe (GtkTreeModel *model,
-					   GtkTreePath *path,
-					   GtkTreeIter *iter,
-					   gpointer data)
+				    GtkTreePath *path,
+				    GtkTreeIter *iter,
+				    gpointer data)
 	{
 	    Playlist *playlist;
 	    
@@ -1155,7 +1159,7 @@ void pm_remove_playlist (Playlist *playlist, gboolean select)
 		return TRUE;
 	    }
 	    return FALSE;
-	};
+	}
     GtkTreeModel *model;
     gboolean have_iter = FALSE;
     GtkTreeIter i;
@@ -1324,6 +1328,8 @@ static void pm_selection_changed_cb (gpointer user_data1, gpointer user_data2)
       st_init (-1, 0);
 
       current_playlist = new_playlist;
+      if (new_playlist->is_spl && new_playlist->splpref.liveupdate)
+	  itdb_spl_update (new_playlist);
       if (new_playlist->members)
       {
 	  GTimeVal time;
@@ -1376,7 +1382,7 @@ static void pm_selection_changed_cb (gpointer user_data1, gpointer user_data2)
 	  time.tv_sec % 3600, time.tv_usec);
 #endif 
   /* make only suitable delete menu items available */
-  display_adjust_delete_menus ();
+  display_adjust_menus ();
 }
 
 /* Callback function called when the selection
@@ -1439,9 +1445,8 @@ void pm_add_itdb (iTunesDB *itdb, gint pos)
     {
 	Playlist *pl = gl_pl->data;
 	g_return_if_fail (pl);
-	if (pl->type == ITDB_PL_TYPE_MPL)
-	     pm_add_playlist (pl, pos);
-	else pm_add_playlist (pl, -1);
+	if (itdb_playlist_is_mpl (pl))     pm_add_playlist (pl, pos);
+	else                               pm_add_playlist (pl, -1);
     }
 }
 
@@ -1606,7 +1611,7 @@ pm_rows_reordered (void)
 	/* get master playlist */
 	gtk_tree_model_get (tm, &parent, PM_COLUMN_PLAYLIST, &pl, -1); 
 	g_return_if_fail (pl);
-	g_return_if_fail (pl->type == ITDB_PL_TYPE_MPL);
+	g_return_if_fail (itdb_playlist_is_mpl (pl));
 	itdb = pl->itdb;
 	g_return_if_fail (itdb);
 
@@ -1621,7 +1626,7 @@ pm_rows_reordered (void)
 	    if (itdb_playlist_by_nr (itdb, pos) != pl)
 	    {
 		/* move the playlist to indicated position */
-		g_return_if_fail (pl->type != ITDB_PL_TYPE_MPL);
+		g_return_if_fail (!itdb_playlist_is_mpl (pl));
 		itdb_playlist_move (pl, pos);
 		data_changed (itdb);
 	    }
@@ -1658,8 +1663,8 @@ gint pm_data_compare_func (GtkTreeModel *model,
   /* We make sure that the master playlist always stays on top */
   if (order == GTK_SORT_ASCENDING) corr = +1;
   else                             corr = -1;
-  if (playlist1->type == ITDB_PL_TYPE_MPL) return (-corr);
-  if (playlist2->type == ITDB_PL_TYPE_MPL) return (corr);
+  if (itdb_playlist_is_mpl (playlist1)) return (-corr);
+  if (itdb_playlist_is_mpl (playlist2)) return (corr);
 
   /* compare the two entries */
   return compare_string (playlist1->name, playlist2->name);
@@ -1686,7 +1691,7 @@ pm_cell_edited (GtkCellRendererText *renderer,
       g_return_if_reached ();
   }
 
-  column = (gint)g_object_get_data (G_OBJECT (renderer), "column");
+  column = GPOINTER_TO_INT ( g_object_get_data (G_OBJECT (renderer), "column") );
   gtk_tree_model_get (model, &iter, column, &playlist, -1);
   g_return_if_fail (playlist);
 
@@ -1727,28 +1732,42 @@ static void pm_cell_data_func (GtkTreeViewColumn *tree_column,
   g_return_if_fail (model);
   g_return_if_fail (iter);
 
-  column = (gint)g_object_get_data (G_OBJECT (renderer), "column");
+  column = GPOINTER_TO_INT ( g_object_get_data (G_OBJECT (renderer), "column") );
   gtk_tree_model_get (model, iter, column, &playlist, -1);
   g_return_if_fail (playlist);
 
   switch (column)
     {  /* We only have one column, so this code is overkill... */
     case PM_COLUMN_PLAYLIST: 
-	if (playlist->type == ITDB_PL_TYPE_MPL)
+	if (itdb_playlist_is_mpl (playlist))
 	{   /* mark MPL */
 	    g_object_set (G_OBJECT (renderer),
 			  "text", playlist->name, 
 			  "editable", TRUE,
 			  "weight", PANGO_WEIGHT_BOLD,
+			  "style", PANGO_STYLE_NORMAL,
 			  NULL);
 	}
 	else
 	{
-	    g_object_set (G_OBJECT (renderer),
-			  "text", playlist->name, 
-			  "editable", TRUE,
-			  "weight", PANGO_WEIGHT_NORMAL,
-			  NULL);
+	    if (itdb_playlist_is_podcasts (playlist))
+	    {
+		g_object_set (G_OBJECT (renderer),
+			      "text", playlist->name, 
+			      "editable", TRUE,
+			      "weight", PANGO_WEIGHT_SEMIBOLD,
+			      "style", PANGO_STYLE_ITALIC,
+			      NULL);
+	    }
+	    else
+	    {
+		g_object_set (G_OBJECT (renderer),
+			      "text", playlist->name, 
+			      "editable", TRUE,
+			      "weight", PANGO_WEIGHT_NORMAL,
+			      "style", PANGO_STYLE_NORMAL,
+			      NULL);
+	    }
 	}
 	break;
     }
@@ -1769,7 +1788,7 @@ static void pm_cell_data_func_pix (GtkTreeViewColumn *tree_column,
   g_return_if_fail (model);
   g_return_if_fail (iter);
 
-  column = (gint)g_object_get_data (G_OBJECT (renderer), "column");
+  column = GPOINTER_TO_INT ( g_object_get_data (G_OBJECT (renderer), "column") );
   gtk_tree_model_get (model, iter, column, &playlist, -1);
   g_return_if_fail (playlist);
 
@@ -1779,12 +1798,12 @@ static void pm_cell_data_func_pix (GtkTreeViewColumn *tree_column,
 	if (playlist->is_spl)
 	{
 	    g_object_set (G_OBJECT (renderer),
-			  "stock-id", "gtk-properties", NULL);
+			  "stock-id", GTK_STOCK_PROPERTIES, NULL);
 	}
-	else if (playlist->type != ITDB_PL_TYPE_MPL)
+	else if (!itdb_playlist_is_mpl (playlist))
 	{
 	    g_object_set (G_OBJECT (renderer),
-			  "stock-id", "gtk-justify-left", NULL);
+			  "stock-id", GTK_STOCK_JUSTIFY_LEFT, NULL);
 	}
 	else
 	{
@@ -1878,7 +1897,7 @@ void pm_create_treeview (void)
   GtkWidget *playlist_window;
   GtkWidget *tree;
 
-  playlist_window = glade_xml_get_widget (main_window_xml, "playlist_window");
+  playlist_window = gtkpod_xml_get_widget (main_window_xml, "playlist_window");
   g_return_if_fail (playlist_window);
 
   /* destroy old treeview */
