@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-07-02 00:38:24 jcs>
+/* Time-stamp: <2005-12-03 20:53:34 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display_songs.c,v 1.81 2005/07/01 16:53:43 jcsjcs Exp $
+|  $Id: display_songs.c,v 1.94 2005/12/03 16:36:26 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -56,9 +56,10 @@ static GtkTreeViewColumn *tm_columns[TM_NUM_COLUMNS];
 static const gint READOUT_COL = 0;
 
 /* compare function to be used for string comparisons */
-static gint (*string_compare_func) (gchar *str1, gchar *str2) = compare_string;
+static gint (*string_compare_func) (const gchar *str1, const gchar *str2) = compare_string;
 
 static GtkTreeViewColumn *tm_add_column (TM_item tm_item, gint position);
+static TM_item tm_lookup_col_id (GtkTreeViewColumn *column);
 
 /* Drag and drop definitions */
 static GtkTargetEntry tm_drag_types [] = {
@@ -75,7 +76,8 @@ static GtkTargetEntry tm_drop_types [] = {
     { "STRING", 0, DND_TEXT_PLAIN }
 };
 
-
+/* prefs strings */
+const gchar *TM_PREFS_SEARCH_COLUMN = "tm_prefs_search_column";
 
 
 /* ---------------------------------------------------------------- */
@@ -259,7 +261,7 @@ static void tm_drag_data_delete (GtkWidget *widget,
     if (num == 0) return;
 
     /* Check if we really have to delete the tracks */
-    if (pl->type == ITDB_PL_TYPE_NORM)
+    if (!itdb_playlist_is_mpl (pl))
     {	/* get list of selected tracks */
 	GString *reply = g_string_sized_new (2000);
 	gchar *str;
@@ -609,7 +611,6 @@ on_track_treeview_key_release_event     (GtkWidget       *widget,
     return FALSE;
 }
 
-
 /* ---------------------------------------------------------------- */
 /* Section for track display                                        */
 /* ---------------------------------------------------------------- */
@@ -776,44 +777,6 @@ GList *gtk_tree_selection_get_selected_rows (GtkTreeSelection *selection,
 #endif
 
 
-/* Returns a string to be displayed in track_nr/tracks column */
-/* You must not free or modify the returned string */
-static const gchar *display_get_track_string (Track *track)
-{
-    static gchar str[20];
-
-    /* Erase string */
-    str[0] = 0;
-    if (track)
-    {
-	if (track->tracks == 0)
-	    snprintf (str, 20, "%d", track->track_nr);
-	else
-	    snprintf (str, 20, "%d/%d", track->track_nr, track->tracks);
-    }
-    return str;
-}
-
-
-/* Returns a string to be displayed in disk_nr/disks column */
-/* You must not free or modify the returned string */
-static const gchar *display_get_disk_string (Track *track)
-{
-    static gchar str[20];
-
-    /* Erase string */
-    str[0] = 0;
-    if (track)
-    {
-	if (track->cds == 0)
-	    snprintf (str, 20, "%d", track->cd_nr);
-	else
-	    snprintf (str, 20, "%d/%d", track->cd_nr, track->cds);
-    }
-    return str;
-}
-
-
 /* Called when editable cell is being edited. Stores new data to the
    track list. ID3 tags in the corresponding files are updated as
    well, if activated in the pref settings */
@@ -829,7 +792,6 @@ tm_cell_edited (GtkCellRendererText *renderer,
   gboolean multi_edit;
   gint sel_rows_num;
   GList *row_list, *row_node, *first;
-  time_t t;
 
 
   column = (TM_item) g_object_get_data(G_OBJECT(renderer), "column");
@@ -856,8 +818,7 @@ tm_cell_edited (GtkCellRendererText *renderer,
      ExtraTrackData *etr;
      gboolean changed;
      GtkTreeIter iter;
-     gint32 nr;
-     gchar **itemp_utf8, *str;
+     gchar *str;
 
      gtk_tree_model_get_iter(model, &iter, (GtkTreePath *) row_node->data);
      gtk_tree_model_get(model, &iter, READOUT_COL, &track, -1);
@@ -875,133 +836,49 @@ tm_cell_edited (GtkCellRendererText *renderer,
      case TM_COLUMN_ARTIST:
      case TM_COLUMN_GENRE:
      case TM_COLUMN_COMPOSER:
-     case TM_COLUMN_FDESC:
+     case TM_COLUMN_COMMENT:
+     case TM_COLUMN_FILETYPE:
      case TM_COLUMN_GROUPING:
-        itemp_utf8 = track_get_item_pointer (track, TM_to_T (column));
-        if (g_utf8_collate (*itemp_utf8, new_text) != 0)
-        {
-           g_free (*itemp_utf8);
-           *itemp_utf8 = g_strdup (new_text);
-           changed = TRUE;
-        }
-        break;
+     case TM_COLUMN_CATEGORY:
+     case TM_COLUMN_DESCRIPTION:
+     case TM_COLUMN_PODCASTURL:
+     case TM_COLUMN_PODCASTRSS:
+     case TM_COLUMN_SUBTITLE:
      case TM_COLUMN_TRACK_NR:
-        nr = atoi (new_text);
-        if ((nr >= 0) && (nr != track->track_nr))
-        {
-           track->track_nr = nr;
-           changed = TRUE;
-        }
-	str = strrchr (new_text, '/');
-	if (str)
-	{
-	    nr = atoi (str+1);
-	    if ((nr >= 0) && (nr != track->tracks))
-	    {
-		track->tracks = nr;
-		changed = TRUE;
-	    }
-	}
-	g_object_set (G_OBJECT (renderer),
-		      "text", display_get_track_string (track), NULL);
-        break;
+     case TM_COLUMN_TRACKLEN:
      case TM_COLUMN_CD_NR:
-        nr = atoi (new_text);
-        if ((nr >= 0) && (nr != track->cd_nr))
-        {
-           track->cd_nr = nr;
-           changed = TRUE;
-        }
-	str = strrchr (new_text, '/');
-	if (str)
-	{
-	    nr = atoi (str+1);
-	    if ((nr >= 0) && (nr != track->cds))
-	    {
-		track->cds = nr;
-		changed = TRUE;
-	    }
-	}
-	g_object_set (G_OBJECT (renderer),
-		      "text", display_get_disk_string (track), NULL);
-        break;
      case TM_COLUMN_YEAR:
-        nr = atoi (new_text);
-        if ((nr >= 0) && (nr != track->year))
-        {
-	   g_free (etr->year_str);
-	   etr->year_str = g_strdup_printf ("%d", nr);
-           track->year = nr;
-           changed = TRUE;
-        }
-        break;
      case TM_COLUMN_PLAYCOUNT:
-        nr = atoi (new_text);
-        if ((nr >= 0) && (nr != track->playcount))
-        {
-           track->playcount = nr;
-           changed = TRUE;
-        }
-        break;
      case TM_COLUMN_RATING:
-        nr = atoi (new_text);
-        if ((nr >= 0) && (nr <= 5) && (nr != track->rating))
-        {
-           track->rating = nr*RATING_STEP;
-           changed = TRUE;
-        }
-        break;
      case TM_COLUMN_TIME_ADDED:
      case TM_COLUMN_TIME_PLAYED:
      case TM_COLUMN_TIME_MODIFIED:
-	 t = time_string_to_time (new_text);
-	 if ((t != -1) && (t != time_get_time (track, column)))
+     case TM_COLUMN_TIME_RELEASED:
+     case TM_COLUMN_VOLUME:
+     case TM_COLUMN_SOUNDCHECK:
+     case TM_COLUMN_BITRATE:
+     case TM_COLUMN_SAMPLERATE:
+     case TM_COLUMN_BPM:
+	 changed = track_set_text (track, new_text, TM_to_T (column));
+	 
+	 /* redisplay some items to be on the safe side */
+	 switch (column)
 	 {
-	     time_set_time (track, t, column);
-	     changed = TRUE;
+	 case TM_COLUMN_TRACK_NR:
+	 case TM_COLUMN_CD_NR:
+	 case TM_COLUMN_TRACKLEN:
+	 case TM_COLUMN_TIME_ADDED:
+	 case TM_COLUMN_TIME_PLAYED:
+	 case TM_COLUMN_TIME_MODIFIED:
+	 case TM_COLUMN_TIME_RELEASED:
+	     str = track_get_text (track, TM_to_T (column));
+	     g_object_set (G_OBJECT (renderer), "text", str, NULL);
+	     g_free (str);
+	     break;
+	 default:
+	     break;
 	 }
 	 break;
-     case TM_COLUMN_VOLUME:
-        nr = atoi (new_text);
-        if (nr != track->volume)
-        {
-	    track->volume = nr;
-	    changed = TRUE;
-        }
-        break;
-     case TM_COLUMN_SOUNDCHECK:
-	nr = replaygain_to_soundcheck (atof (new_text));
-/* 	printf("%d : %f\n", nr, atof (new_text)); */
-        if (nr != track->soundcheck)
-        {
-	    track->soundcheck = nr;
-	    changed = TRUE;
-        }
-        break;
-     case TM_COLUMN_BITRATE:
-        nr = atoi (new_text);
-        if (nr != track->bitrate)
-        {
-	    track->bitrate = nr;
-	    changed = TRUE;
-        }
-        break;
-     case TM_COLUMN_SAMPLERATE:
-        nr = atoi (new_text);
-        if (nr != track->samplerate)
-        {
-	    track->samplerate = nr;
-	    changed = TRUE;
-        }
-        break;
-     case TM_COLUMN_BPM:
-        nr = atoi (new_text);
-        if (nr != track->BPM)
-        {
-	    track->BPM = nr;
-	    changed = TRUE;
-        }
-        break;
      default:
         g_warning ("Programming error: tm_cell_edited: unknown track cell (%d) edited\n", column);
         break;
@@ -1050,11 +927,12 @@ static void tm_cell_data_func (GtkTreeViewColumn *tree_column,
   ExtraTrackData *etr;
   iTunesDB *itdb;
   TM_item column;
-  gchar text[21];
-  gchar *buf = NULL;
-  gchar *item_utf8 = NULL;
+  gchar *text;
 
   column = (TM_item)g_object_get_data (G_OBJECT (renderer), "column");
+
+  g_return_if_fail ((column >= 0) && (column < TM_NUM_COLUMNS));
+
   gtk_tree_model_get (model, iter, READOUT_COL, &track, -1);
   g_return_if_fail (track);
   etr = track->userdata;
@@ -1062,6 +940,8 @@ static void tm_cell_data_func (GtkTreeViewColumn *tree_column,
   itdb = track->itdb;
   g_return_if_fail (itdb);
 
+  text = track_get_text (track, TM_to_T (column));
+
   switch (column)
   {
   case TM_COLUMN_TITLE:
@@ -1069,151 +949,67 @@ static void tm_cell_data_func (GtkTreeViewColumn *tree_column,
   case TM_COLUMN_ALBUM:
   case TM_COLUMN_GENRE:
   case TM_COLUMN_COMPOSER:
-  case TM_COLUMN_FDESC:
+  case TM_COLUMN_COMMENT:
+  case TM_COLUMN_FILETYPE:
   case TM_COLUMN_GROUPING:
-      item_utf8 = track_get_item (track, TM_to_T (column));
+  case TM_COLUMN_CATEGORY:
+  case TM_COLUMN_DESCRIPTION:
+  case TM_COLUMN_PODCASTURL:
+  case TM_COLUMN_PODCASTRSS:
+  case TM_COLUMN_SUBTITLE:
+  case TM_COLUMN_TIME_PLAYED:
+  case TM_COLUMN_TIME_MODIFIED:
+  case TM_COLUMN_TIME_ADDED:
+  case TM_COLUMN_TIME_RELEASED:
       g_object_set (G_OBJECT (renderer),
-		    "text", item_utf8,
-		    "editable", TRUE, NULL);
+		    "text", text,
+		    "editable", TRUE,
+		    "xalign", 0.0, NULL);
       break;
   case TM_COLUMN_TRACK_NR:
-      g_object_set (G_OBJECT (renderer),
-		    "text", display_get_track_string (track),
-		    "editable", TRUE,
-		    "xalign", 1.0, NULL);
-      break;
   case TM_COLUMN_CD_NR:
+  case TM_COLUMN_BITRATE:
+  case TM_COLUMN_SAMPLERATE:
+  case TM_COLUMN_BPM:
+  case TM_COLUMN_PLAYCOUNT:
+  case TM_COLUMN_YEAR:
+  case TM_COLUMN_RATING:
+  case TM_COLUMN_VOLUME:
+  case TM_COLUMN_SOUNDCHECK:
+  case TM_COLUMN_TRACKLEN:
       g_object_set (G_OBJECT (renderer),
-		    "text", display_get_disk_string (track),
+		    "text", text,
 		    "editable", TRUE,
 		    "xalign", 1.0, NULL);
       break;
   case TM_COLUMN_IPOD_ID:
-      if ((track->id != -1) &&
-	  itdb->usertype & GP_ITDB_TYPE_IPOD)
-      {
-	  snprintf (text, 20, "%d", track->id);
-	  g_object_set (G_OBJECT (renderer),
-			"text", text,
-			"xalign", 1.0, NULL);
-      }
-      else
-      {
-	  g_object_set (G_OBJECT (renderer),
-			"text", "--",
-			"xalign", 1.0, NULL);
-      }
+  case TM_COLUMN_SIZE:
+      g_object_set (G_OBJECT (renderer),
+		    "text", text,
+		    "editable", FALSE,
+		    "xalign", 1.0, NULL);
       break;
   case TM_COLUMN_PC_PATH:
-      g_object_set (G_OBJECT (renderer), "text", etr->pc_path_utf8, NULL);
-      break;
   case TM_COLUMN_IPOD_PATH:
-      if (itdb->usertype & GP_ITDB_TYPE_IPOD)
-      {
-	  g_object_set (G_OBJECT (renderer), "text",
-			track->ipod_path, NULL);
-      }
-      if (itdb->usertype & GP_ITDB_TYPE_LOCAL)
-      {
-	  g_object_set (G_OBJECT (renderer), "text",
-			_("Local Database"), NULL);
-      }
+      g_object_set (G_OBJECT (renderer),
+		    "text", text,
+		    "editable", FALSE,
+		    "xalign", 0.0, NULL);
       break;
   case TM_COLUMN_TRANSFERRED:
-      g_object_set (G_OBJECT (renderer), "active", track->transferred, NULL);
+      g_object_set (G_OBJECT (renderer),
+		    "active", track->transferred,
+		    "activatable", FALSE, NULL);
       break;
   case TM_COLUMN_COMPILATION:
       g_object_set (G_OBJECT (renderer),
 		    "active", track->compilation,
 		    "activatable", TRUE, NULL);
       break;
-  case TM_COLUMN_SIZE:
-      snprintf (text, 20, "%d", track->size);
-      g_object_set (G_OBJECT (renderer),
-		    "text", text,
-		    "xalign", 1.0, NULL);
-      break;
-  case TM_COLUMN_TRACKLEN:
-      snprintf (text, 20, "%d:%02d", track->tracklen/60000,
-                                     (track->tracklen/1000)%60);
-      g_object_set (G_OBJECT (renderer),
-		    "text", text,
-		    "xalign", 1.0, NULL);
-      break;
-  case TM_COLUMN_BITRATE:
-      snprintf (text, 20, "%dk", track->bitrate);
-      g_object_set (G_OBJECT (renderer),
-		    "text", text,
-		    "editable", TRUE,
-		    "xalign", 1.0, NULL);
-      break;
-  case TM_COLUMN_SAMPLERATE:
-      snprintf (text, 20, "%d", track->samplerate);
-      g_object_set (G_OBJECT (renderer),
-		    "text", text,
-		    "editable", TRUE,
-		    "xalign", 1.0, NULL);
-      break;
-  case TM_COLUMN_BPM:
-      snprintf (text, 20, "%d", track->BPM);
-      g_object_set (G_OBJECT (renderer),
-		    "text", text,
-		    "editable", TRUE,
-		    "xalign", 1.0, NULL);
-      break;
-  case TM_COLUMN_PLAYCOUNT:
-      snprintf (text, 20, "%d", track->playcount);
-      g_object_set (G_OBJECT (renderer),
-		    "text", text,
-		    "editable", TRUE,
-		    "xalign", 1.0, NULL);
-      break;
-  case TM_COLUMN_YEAR:
-      snprintf (text, 20, "%d", track->year);
-      g_object_set (G_OBJECT (renderer),
-		    "text", text,
-		    "editable", TRUE,
-		    "xalign", 1.0, NULL);
-      break;
-  case TM_COLUMN_RATING:
-      snprintf (text, 20, "%d", track->rating/RATING_STEP);
-      g_object_set (G_OBJECT (renderer),
-		    "text", text,
-		    "editable", TRUE,
-		    "xalign", 1.0, NULL);
-      break;
-  case TM_COLUMN_TIME_PLAYED:
-  case TM_COLUMN_TIME_MODIFIED:
-  case TM_COLUMN_TIME_ADDED:
-      buf = time_field_to_string (track, column);
-      g_object_set (G_OBJECT (renderer),
-		    "text", buf,
- 		    "editable", TRUE,
-		    "xalign", 0.0, NULL);
-      C_FREE (buf);
-      break;
-  case TM_COLUMN_VOLUME:
-      snprintf (text, 20, "%d", track->volume);
-      g_object_set (G_OBJECT (renderer),
-		    "text", text,
-		    "editable", TRUE,
-		    "xalign", 1.0, NULL);
-      break;
-  case TM_COLUMN_SOUNDCHECK:
-/*       printf ("%p:%f : %d\n", track, */
-/* 	      soundcheck_to_replaygain (track->soundcheck), */
-/* 	      track->soundcheck); */
-      snprintf (text, 20, "%0.2f",
-		soundcheck_to_replaygain (track->soundcheck));
-      g_object_set (G_OBJECT (renderer),
-		    "text", text,
-		    "editable", TRUE,
-		    "xalign", 1.0, NULL);
-      break;
-  default:
-      g_warning ("Programming error: unknown column in tm_cell_data_func: %d\n", column);
+  case TM_NUM_COLUMNS:
       break;
   }
+  g_free (text);
 }
 
 
@@ -1247,7 +1043,7 @@ tm_cell_toggled (GtkCellRendererToggle *renderer,
 
   first = g_list_first (row_list);
 
-
+  /* active will show the old state -- before the toggle */
   g_object_get (G_OBJECT (renderer), "active", &active, NULL);
 
   for (row_node = first;
@@ -1289,6 +1085,13 @@ tm_cell_toggled (GtkCellRendererToggle *renderer,
 /*        pm_track_changed (track);  notify playlist model... -- not
  *        necessary here because only the track model is affected */
         data_changed (track->itdb);  /* indicate that data has changed */
+        
+        /* If the changed column is the compilation flag update the file
+           if required */
+        if (column == TM_COLUMN_COMPILATION)
+           if (prefs_get_id3_write())
+              write_tags_to_file (track);
+        
      }
      while (widgets_blocked && gtk_events_pending ())  gtk_main_iteration ();
   }
@@ -1360,7 +1163,7 @@ tm_get_nr_of_tracks(void)
 
 static gint comp_int (gconstpointer a, gconstpointer b)
 {
-    return ((gint)a)-((gint)b);
+    return (GPOINTER_TO_INT(a)-(GPOINTER_TO_INT(b)));
 }
 
 
@@ -1407,7 +1210,7 @@ tm_rows_reordered (void)
 	       happen if track has been added to playlist more than
 	       once */
 	    while ((old_position != -1) &&
-		   g_list_find (old_pos_l, (gpointer)old_position))
+		   g_list_find (old_pos_l, GINT_TO_POINTER(old_position)))
 	    {  /* find next occurence */
 		GList *link;
 		gint next;
@@ -1418,7 +1221,7 @@ tm_rows_reordered (void)
 	    }
 	    /* we make a sorted list of the old positions */
 	    old_pos_l = g_list_insert_sorted (old_pos_l,
-					      (gpointer)old_position,
+					      GINT_TO_POINTER(old_position),
 					      comp_int);
 	    valid = gtk_tree_model_iter_next (tm, &i);
 	}
@@ -1427,7 +1230,7 @@ tm_rows_reordered (void)
 	while (nlp && olp)
 	{
 	    GList *old_link;
-	    gint position = (gint)olp->data;
+	    guint position = GPOINTER_TO_INT(olp->data);
 
 	    /* if position == -1 one of the tracks in the track view
 	       could not be found in the selected playlist -> stop! */
@@ -1468,7 +1271,7 @@ on_trackids_list_foreach ( GtkTreeModel *tm, GtkTreePath *tp,
     GList *l = *((GList**)data);
     gtk_tree_model_get(tm, i, READOUT_COL, &tr, -1);
     g_return_if_fail (tr);
-    l = g_list_append(l, (gpointer)tr->id);
+    l = g_list_append(l, GUINT_TO_POINTER(tr->id));
     *((GList**)data) = l;
 }
 
@@ -1610,9 +1413,15 @@ static gint tm_data_compare (Track *track1, Track *track2,
   case TM_COLUMN_ALBUM:
   case TM_COLUMN_GENRE:
   case TM_COLUMN_COMPOSER:
-  case TM_COLUMN_FDESC:
+  case TM_COLUMN_COMMENT:
+  case TM_COLUMN_FILETYPE:
   case TM_COLUMN_GROUPING:
   case TM_COLUMN_ARTIST:
+  case TM_COLUMN_CATEGORY:
+  case TM_COLUMN_DESCRIPTION:
+  case TM_COLUMN_PODCASTURL:
+  case TM_COLUMN_PODCASTRSS:
+  case TM_COLUMN_SUBTITLE:
       /* string_compare_func is set to either compare_string_fuzzy or
 	 compare_string in tm_sort_column_changed() which is called
 	 once before the comparing begins. */
@@ -1658,8 +1467,10 @@ static gint tm_data_compare (Track *track1, Track *track2,
       break;
   case TM_COLUMN_SIZE:
       cmp = track1->size - track2->size;
+      break;
   case TM_COLUMN_TRACKLEN:
       cmp = track1->tracklen - track2->tracklen;
+      break;
   case TM_COLUMN_BITRATE:
       cmp = track1->bitrate - track2->bitrate;
       break;
@@ -1678,8 +1489,9 @@ static gint tm_data_compare (Track *track1, Track *track2,
   case TM_COLUMN_TIME_ADDED:
   case TM_COLUMN_TIME_PLAYED:
   case TM_COLUMN_TIME_MODIFIED:
-      cmp = COMP (time_get_time (track1, tm_item),
-		  time_get_time (track2, tm_item));
+  case TM_COLUMN_TIME_RELEASED:
+      cmp = COMP (time_get_time (track1, TM_to_T (tm_item)),
+		  time_get_time (track2, TM_to_T (tm_item)));
       break;
   case  TM_COLUMN_VOLUME:
       cmp = track1->volume - track2->volume;
@@ -1710,6 +1522,63 @@ static gint tm_data_compare (Track *track1, Track *track2,
 }
 
 
+/* Function used to compare rows with user's search string */
+gboolean tm_search_equal_func (GtkTreeModel *model,
+			       gint column,
+			       const gchar *key,
+			       GtkTreeIter *iter,
+			       gpointer search_data)
+{
+  Track *track1;
+  gboolean cmp = 0;
+  gtk_tree_model_get (model, iter, READOUT_COL, &track1, -1);
+  switch (column)
+  {
+  case TM_COLUMN_TITLE:
+  case TM_COLUMN_ALBUM:
+  case TM_COLUMN_GENRE:
+  case TM_COLUMN_COMPOSER:
+  case TM_COLUMN_COMMENT:
+  case TM_COLUMN_FILETYPE:
+  case TM_COLUMN_GROUPING:
+  case TM_COLUMN_ARTIST:
+  case TM_COLUMN_CATEGORY:
+  case TM_COLUMN_DESCRIPTION:
+  case TM_COLUMN_PODCASTURL:
+  case TM_COLUMN_PODCASTRSS:
+  case TM_COLUMN_SUBTITLE:
+  case TM_COLUMN_PC_PATH:
+  case TM_COLUMN_YEAR:
+  case TM_COLUMN_IPOD_PATH:
+  case TM_COLUMN_COMPILATION:
+    cmp = (compare_string_start_case_insensitive (
+	  track_get_item (track1, TM_to_T (column)),
+	  key) != 0);
+    break;
+  case TM_COLUMN_TRACK_NR:
+  case TM_COLUMN_IPOD_ID:
+  case TM_COLUMN_TRANSFERRED:
+  case TM_COLUMN_SIZE:
+  case TM_COLUMN_TRACKLEN:
+  case TM_COLUMN_BITRATE:
+  case TM_COLUMN_PLAYCOUNT:
+  case TM_COLUMN_RATING:
+  case TM_COLUMN_TIME_PLAYED:
+  case TM_COLUMN_TIME_MODIFIED:
+  case TM_COLUMN_VOLUME:
+  case TM_COLUMN_CD_NR:
+  case TM_COLUMN_TIME_ADDED:
+  case TM_COLUMN_SOUNDCHECK:
+  case TM_COLUMN_SAMPLERATE:
+  case TM_COLUMN_BPM:
+  case TM_COLUMN_TIME_RELEASED:
+  case TM_NUM_COLUMNS:
+      g_warning ("Programming error: tm_search_equal_func: no sort method for column %d\n", column);
+      break;
+  }
+  return cmp;
+};
+
 /* Function used to compare two cells during sorting (track view) */
 gint tm_data_compare_func (GtkTreeModel *model,
 			GtkTreeIter *a,
@@ -1785,6 +1654,10 @@ static void tm_unsort (void)
 	prefs_set_tm_sort (SORT_NONE);
 	if (!BROKEN_GTK_TREE_SORT)
 	{
+/* no need to comment this out -- searching still works, but for lack
+   of a ctrl-g only the first occurence will be found */
+/*	    gtk_tree_view_set_enable_search (GTK_TREE_VIEW
+ * (track_treeview), FALSE);*/ 
 	    gtk_tree_sortable_set_sort_column_id
 		(GTK_TREE_SORTABLE (model),
 		 GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID,
@@ -1800,6 +1673,59 @@ static void tm_unsort (void)
 }
 
 
+static void tm_set_search_column (gint newcol)
+{
+/*     printf ("track_treeview: %p, col: %d\n", track_treeview, newcol); */
+    g_return_if_fail (track_treeview);
+
+    gtk_tree_view_set_search_column (GTK_TREE_VIEW (track_treeview),
+				       newcol);
+    switch (newcol)
+    {
+    case TM_COLUMN_TITLE:
+    case TM_COLUMN_ALBUM:
+    case TM_COLUMN_GENRE:
+    case TM_COLUMN_COMPOSER:
+    case TM_COLUMN_COMMENT:
+    case TM_COLUMN_FILETYPE:
+    case TM_COLUMN_GROUPING:
+    case TM_COLUMN_ARTIST:
+    case TM_COLUMN_CATEGORY:
+    case TM_COLUMN_DESCRIPTION:
+    case TM_COLUMN_PODCASTURL:
+    case TM_COLUMN_PODCASTRSS:
+    case TM_COLUMN_SUBTITLE:
+    case TM_COLUMN_PC_PATH:
+    case TM_COLUMN_YEAR:
+    case TM_COLUMN_IPOD_PATH:
+    case TM_COLUMN_COMPILATION:
+	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (track_treeview), TRUE);
+	break;
+    case TM_COLUMN_TRACK_NR:
+    case TM_COLUMN_IPOD_ID:
+    case TM_COLUMN_TRANSFERRED:
+    case TM_COLUMN_SIZE:
+    case TM_COLUMN_TRACKLEN:
+    case TM_COLUMN_BITRATE:
+    case TM_COLUMN_PLAYCOUNT:
+    case TM_COLUMN_RATING:
+    case TM_COLUMN_TIME_PLAYED:
+    case TM_COLUMN_TIME_MODIFIED:
+    case TM_COLUMN_VOLUME:
+    case TM_COLUMN_CD_NR:
+    case TM_COLUMN_TIME_ADDED:
+    case TM_COLUMN_SOUNDCHECK:
+    case TM_COLUMN_SAMPLERATE:
+    case TM_COLUMN_BPM:
+    case TM_COLUMN_TIME_RELEASED:
+    case TM_NUM_COLUMNS:
+	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (track_treeview), FALSE);
+	break;
+    }
+    prefs_set_int_value (TM_PREFS_SEARCH_COLUMN, newcol);
+}
+
+
 /* This is called before when changing the sort order or the sort
    column, and before doing the sorting */
 static void tm_sort_column_changed (GtkTreeSortable *ts,
@@ -1846,6 +1772,9 @@ static void tm_sort_column_changed (GtkTreeSortable *ts,
 	prefs_set_tm_sort (order);
     }
     prefs_set_tm_sortcol (newcol);
+
+    tm_set_search_column (newcol);
+
     if(prefs_get_tm_autostore ())  tm_rows_reordered ();
     sort_window_update ();
 
@@ -1909,21 +1838,20 @@ void tm_sort (TM_item col, GtkSortType order)
 /* Add one column at position @pos. This code is used over and over
    by tm_add_column() -- therefore I put it into a separate function */
 static GtkTreeViewColumn *tm_add_text_column (TM_item col_id,
-					      gchar *name,
+					      const gchar *name,
 					      GtkCellRenderer *renderer,
-					      gboolean editable,
 					      gint pos)
 {
     GtkTreeViewColumn *column;
     GtkTreeModel *model = gtk_tree_view_get_model (track_treeview);
 
-    if (!renderer)    renderer = gtk_cell_renderer_text_new ();
-    if (editable)
-    {
+    if (!renderer)
+    {   /* text renderer -- editable/not editable is done in
+	   tm_cell_data_func() */
+	renderer = gtk_cell_renderer_text_new ();
 	g_signal_connect (G_OBJECT (renderer), "edited",
 			  G_CALLBACK (tm_cell_edited), model);
     }
-    g_object_set_data (G_OBJECT (renderer), "editable", (gint *)editable);
     g_object_set_data (G_OBJECT (renderer), "column", (gint *)col_id);
     column = gtk_tree_view_column_new ();
     gtk_tree_view_column_set_title (column, name);
@@ -1951,8 +1879,7 @@ static GtkTreeViewColumn *tm_add_column (TM_item tm_item, gint pos)
 {
   GtkTreeModel *model = gtk_tree_view_get_model (track_treeview);
   GtkTreeViewColumn *col = NULL;
-  gchar *text = NULL;
-  gboolean editable = TRUE;          /* default */
+  const gchar *text = NULL;
   GtkCellRenderer *renderer = NULL;  /* default */
   GtkTooltips *tt;
 
@@ -1974,12 +1901,25 @@ static GtkTreeViewColumn *tm_add_column (TM_item tm_item, gint pos)
   case TM_COLUMN_ALBUM:
   case TM_COLUMN_GENRE:
   case TM_COLUMN_COMPOSER:
-  case TM_COLUMN_FDESC:
+  case TM_COLUMN_COMMENT:
+  case TM_COLUMN_FILETYPE:
   case TM_COLUMN_GROUPING:
-  case TM_COLUMN_RATING:
   case TM_COLUMN_BITRATE:
   case TM_COLUMN_SAMPLERATE:
   case TM_COLUMN_BPM:
+  case TM_COLUMN_CATEGORY:
+  case TM_COLUMN_DESCRIPTION:
+  case TM_COLUMN_PODCASTURL:
+  case TM_COLUMN_PODCASTRSS:
+  case TM_COLUMN_SUBTITLE:
+  case TM_COLUMN_PC_PATH:
+  case TM_COLUMN_IPOD_PATH:
+  case TM_COLUMN_SIZE:
+      break;
+  /* for some column names we want to use shorter alternatives to
+     get_tm_string() */
+  case TM_COLUMN_RATING:
+      text = _("Rtng");
       break;
   case TM_COLUMN_TRACK_NR:
       text = _("#");
@@ -1989,28 +1929,17 @@ static GtkTreeViewColumn *tm_add_column (TM_item tm_item, gint pos)
       break;
   case TM_COLUMN_IPOD_ID:
       text = _("ID");
-      editable = FALSE;
-      break;
-  case TM_COLUMN_PC_PATH:
-  case TM_COLUMN_IPOD_PATH:
-      editable = FALSE;
       break;
   case TM_COLUMN_TRANSFERRED:
       text = _("Trnsfrd");
-      editable = FALSE;
       renderer = gtk_cell_renderer_toggle_new ();
       break;
   case TM_COLUMN_COMPILATION:
       text = _("Cmpl");
-      editable = FALSE;
       renderer = gtk_cell_renderer_toggle_new ();
       break;
-  case TM_COLUMN_SIZE:
-      editable = FALSE;
-      break;
   case TM_COLUMN_TRACKLEN:
       text = _("Time");
-      editable = FALSE;
       break;
   case TM_COLUMN_PLAYCOUNT:
       text = _("Plycnt");
@@ -2024,6 +1953,9 @@ static GtkTreeViewColumn *tm_add_column (TM_item tm_item, gint pos)
   case TM_COLUMN_TIME_ADDED:
       text = _("Added");
       break;
+  case TM_COLUMN_TIME_RELEASED:
+      text = _("Released");
+      break;
   case TM_COLUMN_YEAR:
       text = _("Year");
       break;
@@ -2034,9 +1966,10 @@ static GtkTreeViewColumn *tm_add_column (TM_item tm_item, gint pos)
       text = _("Sndchk.");
       break;
   case TM_NUM_COLUMNS:
+      g_return_val_if_reached (NULL);
       break;
   }
-  col = tm_add_text_column (tm_item, text, renderer, editable, pos);
+  col = tm_add_text_column (tm_item, text, renderer, pos);
   if (col && (tm_item == TM_COLUMN_TITLE))
   {
       renderer = gtk_cell_renderer_toggle_new ();
@@ -2117,6 +2050,17 @@ tm_button_press_event(GtkWidget *w, GdkEventButton *e, gpointer data)
 static void
 tm_selection_changed_event(GtkTreeSelection *selection, gpointer data)
 {
+    GtkTreeView *treeview = gtk_tree_selection_get_tree_view (selection);
+    GtkTreePath *path;
+    GtkTreeViewColumn *column;
+    TM_item col_id;
+
+    gtk_tree_view_get_cursor (treeview, &path, &column);
+    if (path)
+    {
+	col_id = tm_lookup_col_id (column);
+	if (col_id != -1)  tm_set_search_column (col_id);
+    }
     info_update_track_view_selected ();
 }
 
@@ -2125,8 +2069,9 @@ tm_selection_changed_event(GtkTreeSelection *selection, gpointer data)
 void tm_create_treeview (void)
 {
   GtkTreeModel *model = NULL;
-  GtkWidget *track_window = glade_xml_get_widget (main_window_xml, "track_window");
+  GtkWidget *track_window = gtkpod_xml_get_widget (main_window_xml, "track_window");
   GtkTreeSelection *select;
+  gint col;
   GtkWidget *stv = gtk_tree_view_new ();
 
   /* create tree view */
@@ -2145,6 +2090,10 @@ void tm_create_treeview (void)
       gtk_list_store_new (1, G_TYPE_POINTER));
   gtk_tree_view_set_model (track_treeview, GTK_TREE_MODEL (model));
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (track_treeview), TRUE);
+  gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (track_treeview), 
+				       tm_search_equal_func,
+				       NULL,
+				       NULL);
   select = gtk_tree_view_get_selection (track_treeview);
   gtk_tree_selection_set_mode (select,
 			       GTK_SELECTION_MULTIPLE);
@@ -2218,6 +2167,18 @@ void tm_create_treeview (void)
   g_signal_connect (G_OBJECT (model), "sort-column-changed",
 		    G_CALLBACK (tm_sort_column_changed),
 		    (gpointer)0);
+
+  /* initialize sorting */
+  tm_sort (prefs_get_tm_sortcol (), prefs_get_tm_sort ());
+  /* set correct column for typeahead */
+  if (prefs_get_int_value (TM_PREFS_SEARCH_COLUMN, &col))
+  {
+      tm_set_search_column (col);
+  }
+  else
+  {   /* reasonable default */
+      tm_set_search_column (TM_COLUMN_TITLE);
+  }
 }
 
 
@@ -2258,6 +2219,23 @@ void tm_update_default_sizes (void)
 }
 
 
+/* get the TM_ITEM column id for @column. Returns -1 if column could
+   not be found */
+static TM_item tm_lookup_col_id (GtkTreeViewColumn *column)
+{
+    gint i;
+
+    if (column)
+    {
+	for (i=0; i<TM_NUM_COLUMNS; ++i)
+	{
+	    if (column == tm_columns[i]) return i;
+	}
+    }
+    return -1;
+}
+
+
 /* Compare function to avoid sorting */
 static gint tm_nosort_comp (GtkTreeModel *model,
 			    GtkTreeIter *a,
