@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-07-16 01:38:51 jcs>
+/* Time-stamp: <2006-02-19 14:35:41 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: file.c,v 1.135 2005/07/15 16:58:40 jcsjcs Exp $
+|  $Id: file.c,v 1.153 2006/02/19 05:46:24 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -54,6 +54,53 @@
 #include "prefs.h"
 #include "wavfile.h"
 
+
+static const gchar *imageext[] =
+{".jpg", ".jpeg", ".png", ".pbm", ".pgm", ".ppm", ".tif", ".tiff",
+ ".gif", NULL};
+
+
+/*------------------------------------------------------------------*\
+ *                                                                  *
+ *      Temporary Video stuff -- move to appropriate files when     *
+ *      properly supported.                                         *
+ *                                                                  *
+\*------------------------------------------------------------------*/
+
+Track *video_get_file_info (gchar *filename)
+{
+    Track *track;
+    track = gp_track_new ();
+    switch (determine_file_type (filename))
+    {
+    case FILE_TYPE_M4V:
+	track->filetype = g_strdup ("M4V video file");
+	break;
+    case FILE_TYPE_MP4:
+	track->filetype = g_strdup ("MP4 video file");
+	break;
+    case FILE_TYPE_MOV:
+	track->filetype = g_strdup ("MOV video file");
+	break;
+    case FILE_TYPE_MPG:
+	track->filetype = g_strdup ("MPG video file");
+	break;
+    case FILE_TYPE_UNKNOWN:
+    case FILE_TYPE_MP3:
+    case FILE_TYPE_M4A:
+    case FILE_TYPE_M4P:
+    case FILE_TYPE_M4B:
+    case FILE_TYPE_WAV:
+    case FILE_TYPE_M3U:
+    case FILE_TYPE_PLS:
+    case FILE_TYPE_IMAGE:
+	g_free (track);
+	g_return_val_if_reached (NULL);
+    }
+    return track;
+}
+
+
 /* Determine the type of a file. 
  *
  * Currently this is done by checking the suffix of the filename. An improved
@@ -62,30 +109,49 @@
  * -jlt
  */
 
-gint determine_file_type (gchar *path)
+FileType determine_file_type (gchar *path)
 {
-	gchar *path_utf8, *suf;
-	gint type = FILE_TYPE_UNKNOWN;
+    gchar *path_utf8, *suf;
+    FileType type = FILE_TYPE_UNKNOWN;
 	
-	g_return_val_if_fail (path, type);
+    g_return_val_if_fail (path, type);
 	
-    	path_utf8 = charset_to_utf8 (path);
-	suf = strrchr (path_utf8, '.');
-	if (suf)
-	{
-	    /* since we are exclusively checking for equality strcasecmp
-	     * should be sufficient */
-	    if (g_strcasecmp (suf, ".mp3") == 0) type = FILE_TYPE_MP3;
-	    else if (g_strcasecmp (suf, ".m4a") == 0) type = FILE_TYPE_M4A;
-	    else if (g_strcasecmp (suf, ".m4p") == 0) type = FILE_TYPE_M4P;
-	    else if (g_strcasecmp (suf, ".m4b") == 0) type = FILE_TYPE_M4B;
-	    else if (g_strcasecmp (suf, ".wav") == 0) type = FILE_TYPE_WAV;
-	    else if (g_strcasecmp (suf, ".m3u") == 0) type = FILE_TYPE_M3U;
-	    else if (g_strcasecmp (suf, ".pls") == 0) type = FILE_TYPE_PLS;
-	}
+    path_utf8 = charset_to_utf8 (path);
+    suf = strrchr (path_utf8, '.');
+    if (suf)
+    {
+	if (g_strcasecmp (suf, ".mp3") == 0) type = FILE_TYPE_MP3;
+	else if (g_strcasecmp (suf, ".m4a") == 0) type = FILE_TYPE_M4A;
+	else if (g_strcasecmp (suf, ".m4p") == 0) type = FILE_TYPE_M4P;
+	else if (g_strcasecmp (suf, ".m4b") == 0) type = FILE_TYPE_M4B;
+	else if (g_strcasecmp (suf, ".wav") == 0) type = FILE_TYPE_WAV;
 
-	g_free(path_utf8);
-	return type;
+	else if (g_strcasecmp (suf, ".m4v") == 0) type = FILE_TYPE_M4V;
+	else if (g_strcasecmp (suf, ".mp4") == 0) type = FILE_TYPE_MP4;
+	else if (g_strcasecmp (suf, ".mov") == 0) type = FILE_TYPE_MOV;
+	else if (g_strcasecmp (suf, ".mpg") == 0) type = FILE_TYPE_MPG;
+	else if (g_strcasecmp (suf, ".mpeg") == 0) type = FILE_TYPE_MPG;
+
+	else if (g_strcasecmp (suf, ".m3u") == 0) type = FILE_TYPE_M3U;
+	else if (g_strcasecmp (suf, ".pls") == 0) type = FILE_TYPE_PLS;
+
+	else
+	{
+	    const gchar **extp=imageext;
+	    while (*extp)
+	    {
+		if (g_strcasecmp (suf, *extp) == 0)
+		{
+		    type = FILE_TYPE_IMAGE;
+		    break;
+		}
+		++extp;
+	    }
+	}
+    }
+
+    g_free(path_utf8);
+    return type;
 }
 
 
@@ -116,7 +182,7 @@ add_playlist_by_filename (iTunesDB *itdb, gchar *plfile,
     gchar *bufp, *plfile_utf8;
     gchar *dirname = NULL, *plname = NULL;
     gchar buf[PATH_MAX];
-    gint type = FILE_TYPE_UNKNOWN; /* type of playlist file */
+    FileType type = FILE_TYPE_UNKNOWN; /* type of playlist file */
     gint line, tracks;
     FILE *fp;
     gboolean error;
@@ -148,6 +214,11 @@ add_playlist_by_filename (iTunesDB *itdb, gchar *plfile,
 	case FILE_TYPE_M4P:
 	case FILE_TYPE_M4B:
 	case FILE_TYPE_WAV:
+	case FILE_TYPE_M4V:
+	case FILE_TYPE_MP4:
+	case FILE_TYPE_MOV:
+	case FILE_TYPE_MPG:
+	case FILE_TYPE_IMAGE:
 	    gtkpod_warning (_("'%s' is a not a known playlist file.\n\n"),
 			      plfile);
 	    g_free(plname);
@@ -157,7 +228,7 @@ add_playlist_by_filename (iTunesDB *itdb, gchar *plfile,
 	    break;
 	case FILE_TYPE_UNKNOWN:
 	    /* assume MISC (M3U like) style */
-	    type = FILE_TYPE_MISC;
+	    type = -2;
 	    break;
 	}
     }
@@ -193,9 +264,8 @@ add_playlist_by_filename (iTunesDB *itdb, gchar *plfile,
 	++line;
 	if (len == 0) continue;   /* skip empty lines */
 	if (bufp[len-1] == 0x0a)  bufp[len-1] = 0;
-	switch (type)
+	if (type == -2)
 	{
-	case FILE_TYPE_MISC:
 	    /* skip whitespace */
 	    while (isspace (*bufp)) ++bufp;
 	    /* assume comments start with ';' or '#' */
@@ -203,6 +273,9 @@ add_playlist_by_filename (iTunesDB *itdb, gchar *plfile,
 	    /* assume the rest of the line is a filename */
 	    filename = concat_dir_if_relative (dirname, bufp);
 	    break;
+	}
+	else switch (type)
+	{
 	case FILE_TYPE_M3U:
 	    /* comments start with '#' */
 	    if (*bufp == '#') break;
@@ -228,6 +301,18 @@ add_playlist_by_filename (iTunesDB *itdb, gchar *plfile,
 		}
 	    }
 	    break;
+	case FILE_TYPE_UNKNOWN:
+	case FILE_TYPE_MP3:
+	case FILE_TYPE_M4A:
+	case FILE_TYPE_M4P:
+	case FILE_TYPE_M4B:
+	case FILE_TYPE_WAV:
+	case FILE_TYPE_M4V:
+	case FILE_TYPE_MP4:
+	case FILE_TYPE_MOV:
+	case FILE_TYPE_MPG:
+	case FILE_TYPE_IMAGE:
+	    break;
 	}
 	if (filename)
 	{
@@ -426,7 +511,7 @@ static gboolean parse_filename_with_template (Track *track,
 #ifdef DEBUG
 		printf ("%s: '%s'\n", token, fnp);
 #endif
-		itm = g_strdup (fnp);
+		itm = g_strstrip (g_strdup (fnp));
 		switch (token[1])
 		{
 		case 'a': /* artist */
@@ -644,7 +729,7 @@ void update_charset_info (Track *track)
 
 /* Copy "new" info read from file to an old Track structure.
    Return value: a pointer to the track the data was copied to. */
-Track *copy_new_info (Track *from, Track *to)
+static Track *copy_new_info (Track *from, Track *to)
 {
     ExtraTrackData *efrom, *eto;
 
@@ -655,16 +740,23 @@ Track *copy_new_info (Track *from, Track *to)
     g_return_val_if_fail (efrom, NULL);
     g_return_val_if_fail (eto, NULL);
 
-    C_FREE (to->album);
-    C_FREE (to->artist);
-    C_FREE (to->title);
-    C_FREE (to->genre);
-    C_FREE (to->comment);
-    C_FREE (to->composer);
-    C_FREE (to->fdesc);
-    C_FREE (eto->pc_path_utf8);
-    C_FREE (eto->pc_path_locale);
-    C_FREE (eto->charset);
+    g_free (to->title);
+    g_free (to->artist);
+    g_free (to->album);
+    g_free (to->genre);
+    g_free (to->composer);
+    g_free (to->comment);
+    g_free (to->filetype);
+    g_free (to->description);
+    g_free (to->podcasturl);
+    g_free (to->podcastrss);
+    g_free (to->subtitle);
+    g_free (eto->pc_path_utf8);
+    g_free (eto->pc_path_locale);
+    g_free (eto->charset);
+    g_free (eto->thumb_path_utf8);
+    g_free (eto->thumb_path_locale);
+    itdb_artwork_free (to->artwork);
     /* copy strings */
     to->album = g_strdup (from->album);
     to->artist = g_strdup (from->artist);
@@ -672,10 +764,16 @@ Track *copy_new_info (Track *from, Track *to)
     to->genre = g_strdup (from->genre);
     to->comment = g_strdup (from->comment);
     to->composer = g_strdup (from->composer);
-    to->fdesc = g_strdup (from->fdesc);
+    to->filetype = g_strdup (from->filetype);
+    to->podcasturl = g_strdup (from->podcasturl);
+    to->podcastrss = g_strdup (from->podcastrss);
+    to->subtitle = g_strdup (from->subtitle);
+    to->description = g_strdup (from->description);
     eto->pc_path_utf8 = g_strdup (efrom->pc_path_utf8);
     eto->pc_path_locale = g_strdup (efrom->pc_path_locale);
     eto->charset = g_strdup (efrom->charset);
+    eto->thumb_path_utf8 = g_strdup (efrom->thumb_path_utf8);
+    eto->thumb_path_locale = g_strdup (efrom->thumb_path_locale);
     to->size = from->size;
     to->tracklen = from->tracklen;
     to->cd_nr = from->cd_nr;
@@ -694,8 +792,12 @@ Track *copy_new_info (Track *from, Track *to)
     to->time_added = from->time_added;
     to->time_modified = from->time_modified;
     to->year = from->year;
+    to->compilation = from->compilation;
     g_free (eto->year_str);
     eto->year_str = g_strdup_printf ("%d", to->year);
+    to->unk208 = from->unk208;
+    /* copy artwork */
+    to->artwork = itdb_artwork_duplicate (from->artwork);
 
     return to;
 }
@@ -755,9 +857,9 @@ gboolean update_mserv_data_from_file (gchar *name, Track *track)
 			&& buff[usernamelen] == (gchar)'=')
 		    {
 			/* found it */
-			track->rating = atoi(&buff[usernamelen+1]) * RATING_STEP;
+			track->rating = atoi(&buff[usernamelen+1]) * ITDB_RATING_STEP;
 			/* printf("found it, = %d\n",
-			   orig_track->rating/RATING_STEP); */
+			   orig_track->rating/ITDB_RATING_STEP); */
 			success = TRUE;
 			break; /* while(fgets(... */
 		    }
@@ -793,15 +895,87 @@ gboolean update_mserv_data_from_file (gchar *name, Track *track)
 }
 
 
+
+/* look for a picture specified by coverart_template  */
+static void add_artwork (Track *tr)
+{
+    ExtraTrackData *etr;
+    const gchar *full_template;
+    gchar **templates, **tplp;
+    gchar *dirname;
+    gchar *filename_local = NULL;
+
+    g_return_if_fail (tr);
+    etr = tr->userdata;
+    g_return_if_fail (etr);
+
+    dirname = g_path_get_dirname (etr->pc_path_utf8);
+
+    full_template = prefs_get_coverart_template ();
+
+    templates = g_strsplit (full_template, ";", 0);
+    tplp = templates;
+    while (*tplp && !filename_local)
+    {
+	gchar *filename_utf8;
+	gchar *fname = get_string_from_template (tr, *tplp, TRUE);
+	if (fname)
+	{
+	    if (strchr (fname, '.') != NULL)
+	    {   /* if fname has an extension, try if it is valid */
+		filename_utf8 = g_build_filename (dirname, fname, NULL);
+		filename_local = charset_from_utf8 (filename_utf8);
+		g_free (filename_utf8);
+		if (!g_file_test (filename_local, G_FILE_TEST_EXISTS))
+		{
+		    g_free (filename_local);
+		    filename_local = NULL;
+		}
+	    }
+	    else
+	    {   /* otherwise try out different extensions */
+		const gchar **extp = imageext;
+		while (*extp && !filename_local)
+		{
+		    gchar *ffname;
+		    ffname = g_strconcat (fname, *extp, NULL);
+		    filename_utf8 = g_build_filename (dirname, ffname, NULL);
+		    g_free (ffname);
+		    filename_local = charset_from_utf8 (filename_utf8);
+		    g_free (filename_utf8);
+		    if (!g_file_test (filename_local, G_FILE_TEST_EXISTS))
+		    {
+			g_free (filename_local);
+			filename_local = NULL;
+		    }
+		    ++extp;
+		}
+	    }
+	}
+	g_free (fname);
+	++tplp;
+    }
+
+    if (filename_local)
+    {
+	gp_track_set_thumbnails (tr, filename_local);
+    }
+
+    g_strfreev (templates);
+}
+
+
+
 /* Fills the supplied @orig_track with data from the file @name. If
  * @or_track is NULL, a new track struct is created. The entries
  * pc_path_utf8 and pc_path_locale are not changed if an entry already
  * exists */
 /* Returns NULL on error, a pointer to the Track otherwise */
-Track *get_track_info_from_file (gchar *name, Track *orig_track)
+static Track *get_track_info_from_file (gchar *name, Track *orig_track)
 {
     Track *track = NULL;
     Track *nti = NULL;
+    FileType filetype;
     gint len;
     gchar *name_utf8 = NULL;
 
@@ -825,23 +999,47 @@ Track *get_track_info_from_file (gchar *name, Track *orig_track)
     len = strlen (name);
     if (len < 4) return NULL;
 
-    switch (determine_file_type(name))
+    filetype = determine_file_type(name);
+    switch (filetype)
     {
     case FILE_TYPE_MP3:
 	nti = mp3_get_file_info (name);
+	/* Set unk208 to audio */
+	if (nti) nti->unk208 = 0x00000001;
 	break;
     case FILE_TYPE_M4A:
     case FILE_TYPE_M4P:
     case FILE_TYPE_M4B:
 	nti = mp4_get_file_info (name);
+	/* Set unk208 to audio */
+	if (nti) nti->unk208 = 0x00000001;
 	break;
     case FILE_TYPE_WAV:
 	nti = wav_get_file_info (name);
+	/* Set unk208 to audio */
+	if (nti) nti->unk208 = 0x00000001;
+	break;
+    case FILE_TYPE_M4V:
+    case FILE_TYPE_MP4:
+	/* I don't know if .m4v and .mp4 can simply be handled like
+	   this. Let's see if someone complains. */
+	nti = mp4_get_file_info (name);
+	if (!nti) video_get_file_info (name);
+	/* Set unk208 to video */
+	if (nti) nti->unk208 = 0x00000002;
+	break;
+    case FILE_TYPE_MOV:
+    case FILE_TYPE_MPG:
+	/* for now treat all the same */
+	nti = video_get_file_info (name);
+	/* Set unk208 to video */
+	if (nti) nti->unk208 = 0x00000002;
 	break;
     case FILE_TYPE_UNKNOWN:
 	gtkpod_warning (_("The following track could not be processed (filetype unknown): '%s'\n"), name_utf8);
 	g_free (name_utf8);
 	return NULL;
+    case FILE_TYPE_IMAGE:
     case FILE_TYPE_M3U:
     case FILE_TYPE_PLS:
 	break;
@@ -877,16 +1075,23 @@ Track *get_track_info_from_file (gchar *name, Track *orig_track)
 		replaygain_to_soundcheck (enti->radio_gain);
 	/* Set unset strings (album...) from filename */
 	set_unset_entries_from_filename (nti);
-	/* Make sure all strings are initialized -- that way we don't
-	   have to worry about it when we are handling the
-	   strings. Also, validate_entries() will fill in the utf16
-	   strings if that hasn't already been done. */
-	/* exception: md5_hash, charset and hostname: these may be NULL. */
+
+	/* Set coverart */
+	if (prefs_get_coverart())
+	    add_artwork (nti);
+
 	/* Set modification date to *now* */
 	nti->time_modified = itdb_time_get_mac_time ();
 	/* Set added date to *now* */
 	nti->time_added = nti->time_modified;
 
+	/* Make sure all strings are initialized -- that way we don't
+	   have to worry about it when we are handling the
+	   strings. Also, validate_entries() will fill in the utf16
+	   strings if that hasn't already been done. */
+	/* exception: md5_hash, charset and hostname: these may be
+	 * NULL. */
+
 	gp_track_validate_entries (nti);
 
 	if (orig_track)
@@ -906,8 +1111,17 @@ Track *get_track_info_from_file (gchar *name, Track *orig_track)
     }
     else
     {
-	gtkpod_warning (_("The following track could not be processed (filetype is known but analysis failed): '%s'\n"), name_utf8);
-    }	
+	switch (filetype)
+	{
+	case FILE_TYPE_IMAGE:
+	case FILE_TYPE_M3U:
+	case FILE_TYPE_PLS:
+	    break;
+	default:
+	    gtkpod_warning (_("The following track could not be processed (filetype is known but analysis failed): '%s'\n"), name_utf8);
+	    break;
+	}
+    }
 
     while (widgets_blocked && gtk_events_pending ())
 	gtk_main_iteration ();
@@ -929,6 +1143,7 @@ Track *get_track_info_from_file (gchar *name, Track *orig_track)
 void update_tracks (GList *selected_tracks)
 {
     GList *gl;
+    iTunesDB *itdb = NULL;
 
     if (selected_tracks == NULL)
     {
@@ -941,11 +1156,23 @@ void update_tracks (GList *selected_tracks)
     {
 	Track *track = gl->data;
 	g_return_if_fail (track);
-	gchar *buf = g_strdup_printf (_("Updating %s"),
-				      get_track_info (track, TRUE));
-	gtkpod_statusbar_message (buf);
-	g_free (buf);
-	update_track_from_file (track->itdb, track);
+
+	/* update_track_from_file() may possibly remove tracks from
+	   the database, so we need to check if the track we are
+	   referencing to is still valid. To do so we first have to
+	   secure a valid pointer to an itdb. Since the first track in
+	   selected_tracks is always valid, we take that one. */
+	if (!itdb)
+	    itdb = track->itdb;
+	g_return_if_fail (itdb);
+	if (g_list_find (itdb->tracks, track))
+	{
+	    gchar *buf = g_strdup_printf (_("Updating %s"),
+					  get_track_info (track, TRUE));
+	    gtkpod_statusbar_message (buf);
+	    g_free (buf);
+	    update_track_from_file (track->itdb, track);
+	}
     }
     release_widgets ();
     /* display log of non-updated tracks */
@@ -1742,8 +1969,8 @@ void update_track_from_file (iTunesDB *itdb, Track *track)
  *                                                                  *
 \*------------------------------------------------------------------*/
 
-/* Append file @name to the list of tracks.
-   @name is in the current locale
+/* Append file @fname to the list of tracks.
+   @fname is in the current locale
    @plitem: if != NULL, add track to plitem as well (unless it's the MPL)
    descend: TRUE:  add directories recursively
             FALSE: add contents of directories passed but don't descend
@@ -1751,7 +1978,7 @@ void update_track_from_file (iTunesDB *itdb, Track *track)
 /* @addtrackfunc: if != NULL this will be called instead of
    "add_track_to_playlist () -- used for dropping tracks at a specific
    position in the track view */
-gboolean add_track_by_filename (iTunesDB *itdb, gchar *name,
+gboolean add_track_by_filename (iTunesDB *itdb, gchar *fname,
 				Playlist *plitem, gboolean descend,
 			       AddTrackFunc addtrackfunc, gpointer data)
 {
@@ -1763,36 +1990,43 @@ gboolean add_track_by_filename (iTunesDB *itdb, gchar *name,
   Playlist *mpl;
   gboolean result = TRUE;
 
-  g_return_val_if_fail (name, FALSE);
+  g_return_val_if_fail (fname, FALSE);
   g_return_val_if_fail (itdb, FALSE);
   mpl = itdb_playlist_mpl (itdb);
   g_return_val_if_fail (mpl, FALSE);
 
   if (!plitem)  plitem = mpl;
 
-  if (g_file_test (name, G_FILE_TEST_IS_DIR))
+  if (g_file_test (fname, G_FILE_TEST_IS_DIR))
   {
-      return add_directory_by_name (itdb, name, plitem, descend, addtrackfunc, data);
+      return add_directory_by_name (itdb, fname, plitem, descend, addtrackfunc, data);
   }
 
   /* check if file is a playlist */
-  switch (determine_file_type(name)) {
-	  case FILE_TYPE_M3U:
-	  case FILE_TYPE_PLS:
-	      if (add_playlist_by_filename (itdb, name, plitem, -1,
-					    addtrackfunc, data))
-		  return TRUE;
-	      return FALSE;
-	  case FILE_TYPE_MP3:
-	  case FILE_TYPE_M4A:
-	  case FILE_TYPE_M4P:
-	  case FILE_TYPE_M4B:
-	  case FILE_TYPE_UNKNOWN:
-		  break;
+  switch (determine_file_type(fname))
+  {
+  case FILE_TYPE_M3U:
+  case FILE_TYPE_PLS:
+      if (add_playlist_by_filename (itdb, fname, plitem, -1,
+				    addtrackfunc, data))
+	  return TRUE;
+      return FALSE;
+  case FILE_TYPE_MP3:
+  case FILE_TYPE_M4A:
+  case FILE_TYPE_M4P:
+  case FILE_TYPE_M4B:
+  case FILE_TYPE_WAV:
+  case FILE_TYPE_M4V:
+  case FILE_TYPE_MP4:
+  case FILE_TYPE_MOV:
+  case FILE_TYPE_MPG:
+  case FILE_TYPE_IMAGE:
+  case FILE_TYPE_UNKNOWN:
+      break;
   }
 
   /* print a message about which file is being processed */
-  basename = g_path_get_basename (name);
+  basename = g_path_get_basename (fname);
   if (basename)
   {
       gchar *bn_utf8 = charset_to_utf8 (basename);
@@ -1804,9 +2038,9 @@ gboolean add_track_by_filename (iTunesDB *itdb, gchar *name,
   C_FREE (basename);
 
   /* Check if there exists already a track with the same filename */
-  oldtrack = gp_track_by_filename (itdb, name);
+  oldtrack = gp_track_by_filename (itdb, fname);
   /* If a track already exists in the database, either update it or
-     just add it to the current playlist (if it doesn't already exist) */
+     just add it to the current playlist (if it's not already there) */
   if (oldtrack)
   {
       if (prefs_get_update_existing ())
@@ -1814,7 +2048,7 @@ gboolean add_track_by_filename (iTunesDB *itdb, gchar *name,
 	  update_track_from_file (itdb, oldtrack);
       }
       /* add to current playlist if it's not already in there */
-      if (plitem->type != ITDB_PL_TYPE_MPL)
+      if (!itdb_playlist_is_mpl (plitem))
       {
 	  if (!itdb_playlist_contains_track (plitem, oldtrack))
 	  {
@@ -1826,9 +2060,8 @@ gboolean add_track_by_filename (iTunesDB *itdb, gchar *name,
       }
   }
   else  /* oldtrack == NULL */
-  {  /* Only read the new track if there doesn't already exist an old
-	track with the same filename in the database */
-      Track *track = get_track_info_from_file (name, NULL);
+  {   /* OK, the same filename does not already exist */
+      Track *track = get_track_info_from_file (fname, NULL);
       if (track)
       {
           Track *added_track = NULL;
@@ -1838,25 +2071,26 @@ gboolean add_track_by_filename (iTunesDB *itdb, gchar *name,
 	  track->id = 0;
 	  track->transferred = FALSE;
 
-	  /* is 'name' on the iPod? */
+	  /* is 'fname' on the iPod? -- if yes mark as transfered, if
+	   * it's in the ipod_control directory */
 	  if (itdb->usertype & GP_ITDB_TYPE_IPOD)
 	  {
 	      g_return_val_if_fail (itdb->mountpoint, FALSE);
-	      if (strstr (name, itdb->mountpoint) == name)
+	      if (strstr (fname, itdb->mountpoint) == fname)
 	      {   /* Yes */
-		  /* is 'name' in the iPod_Control directory? */
-		  gchar *name_i = name + strlen (itdb->mountpoint);
-		  gchar *name_l;
-		  if (*name_i == G_DIR_SEPARATOR) ++name_i;
-		  name_l = g_ascii_strdown (name_i, -1);
-		  if (strstr (name_l, "ipod_control") == name_l)
+		  /* is 'fname' in the iPod_Control directory? */
+		  gchar *fname_i = fname + strlen (itdb->mountpoint);
+		  gchar *fname_l;
+		  if (*fname_i == G_DIR_SEPARATOR) ++fname_i;
+		  fname_l = g_ascii_strdown (fname_i, -1);
+		  if (strstr (fname_l, "ipod_control") == fname_l)
 		  {   /* Yes */
 		      track->transferred = TRUE;
 		      track->ipod_path = g_strdup_printf (
-			  "%c%s", G_DIR_SEPARATOR, name_i);
+			  "%c%s", G_DIR_SEPARATOR, fname_i);
 		      itdb_filename_fs2ipod (track->ipod_path);
 		  }
-		  g_free (name_l);
+		  g_free (fname_l);
 	      }
 	  }
 
@@ -1870,36 +2104,48 @@ gboolean add_track_by_filename (iTunesDB *itdb, gchar *name,
 	  added_track = gp_track_add (itdb, track);
 	  g_return_val_if_fail (added_track, FALSE);
 
-	  if (addtrackfunc)
-	  {
-	      if (plitem->type == ITDB_PL_TYPE_MPL)
-	      {   /* add track to master playlist (if it wasn't a
-		     duplicate */
-		  if (added_track == track)
+	  /* set flags to 'podcast' if adding to podcast list */
+	  if (itdb_playlist_is_podcasts (plitem))
+	      gp_track_set_flags_podcast (added_track);
+
+	  if (itdb_playlist_is_mpl (plitem))
+	  {   /* add track to master playlist if it wasn't a
+		 duplicate */
+	      if (added_track == track)
+	      {
+		  if (addtrackfunc)
 		      addtrackfunc (plitem, added_track, data);
+		  else
+		      gp_playlist_add_track (plitem, added_track, TRUE);
+	      }
+	  }
+	  else
+	  {   /* add track to master playlist if it wasn't a
+	       * duplicate and plitem is not the podcasts playlist
+	       */
+	      if (added_track == track)
+	      {
+		  if (!itdb_playlist_is_podcasts (plitem))
+		      gp_playlist_add_track (mpl, added_track, TRUE);
+	      }
+	      /* add track to specified playlist -- unless adding
+	       * to podcasts list and track already exists there */
+	      if (itdb_playlist_is_podcasts (plitem) &&
+		  g_list_find (plitem->members, added_track))
+	      {
+		  gchar *buf = get_track_info (added_track, FALSE);
+		  gtkpod_warning (_("Podcast already present: '%s'\n\n"), buf);
+		  g_free (buf);
 	      }
 	      else
-	      {   /* add track to master playlist (if it wasn't a
-		   * duplicate) */
-		  if (added_track == track)
-		  {
-		      gp_playlist_add_track (mpl, added_track, TRUE);
-		  }
-		  /* add track to specified playlist */
-		  addtrackfunc (plitem, added_track, data);
+	      {
+		  if (addtrackfunc)
+		      addtrackfunc (plitem, added_track, data);
+		  else
+		      gp_playlist_add_track (plitem, added_track, TRUE);
 	      }
 	  }
-	  else  /* no addtrackfunc */
-	  {
-		  /* add track to master playlist (if it wasn't a
-		   * duplicate) */
-		  if (added_track == track)
-		      gp_playlist_add_track (mpl, added_track,
-					     TRUE);
-		  /* add track to specified playlist, but not to MPL */
-		  if (plitem->type != ITDB_PL_TYPE_MPL)
-		      gp_playlist_add_track (plitem, added_track, TRUE);
-	  }
+
 	  /* indicate that non-transferred files exist */
 	  data_changed (itdb);
 	  ++count;
@@ -1929,23 +2175,37 @@ gboolean add_track_by_filename (iTunesDB *itdb, gchar *name,
 /* Call the correct tag writing function for the filename @name */
 static gboolean file_write_info (gchar *name, Track *track)
 {
-    if (name && track)
+    gchar *buf;
+
+    g_return_val_if_fail (name, FALSE);
+    g_return_val_if_fail (track, FALSE);
+
+    switch (determine_file_type(name))
     {
-        switch (determine_file_type(name)) {
-	    case FILE_TYPE_MP3:
-		return mp3_write_file_info (name, track);
-	    case FILE_TYPE_M4A:
-	    case FILE_TYPE_M4P:
-	    case FILE_TYPE_M4B:
-		return mp4_write_file_info (name, track);
-	    case FILE_TYPE_WAV:
-		return wav_write_file_info (name, track);
-	    case FILE_TYPE_M3U:
-	    case FILE_TYPE_PLS:
-	    case FILE_TYPE_UNKNOWN:
-		break;
-	}
+    case FILE_TYPE_MP3:
+	return mp3_write_file_info (name, track);
+    case FILE_TYPE_M4A:
+    case FILE_TYPE_M4P:
+    case FILE_TYPE_M4B:
+	return mp4_write_file_info (name, track);
+    case FILE_TYPE_WAV:
+	return wav_write_file_info (name, track);
+    case FILE_TYPE_M4V:
+    case FILE_TYPE_MP4:
+    case FILE_TYPE_MOV:
+    case FILE_TYPE_MPG:
+	buf = get_track_info (track, FALSE);
+	gtkpod_warning (_("Writing to video files not yet supported (%s).\n\n"),
+			buf);
+	g_free (buf);
+	break;
+    case FILE_TYPE_M3U:
+    case FILE_TYPE_PLS:
+    case FILE_TYPE_IMAGE:
+    case FILE_TYPE_UNKNOWN:
+	break;
     }
+
     return FALSE;
 }
 
@@ -2044,7 +2304,8 @@ gchar* get_file_name(Track *tr)
     etr = tr->userdata;
     g_return_val_if_fail (etr, result);
 
-    result = get_file_name_on_ipod (tr);
+    if (tr->itdb)
+	result = get_file_name_on_ipod (tr);
     if(!result &&
        (etr->pc_path_locale) && (strlen(etr->pc_path_locale) > 0))
     {
@@ -2067,9 +2328,7 @@ gchar *get_file_name_on_ipod (Track *tr)
 
     if(tr &&  !prefs_get_offline ())
     {
-	gchar *mount = charset_from_utf8 (prefs_get_ipod_mount ());
-	result = itdb_filename_on_ipod (mount, tr);
-	g_free (mount);
+	result = itdb_filename_on_ipod (tr);
     }
     return(result);
 }
@@ -2412,9 +2671,14 @@ void parse_offline_playcount (void)
 /* Set the gain value in @track. Return value: TRUE, if gain could be set */
 gboolean get_gain (Track *track) 
 {
-    gchar *path = get_file_name_verified (track);
+    gchar *path;
+    gchar *buf;
     gboolean result = FALSE;
 
+    g_return_val_if_fail (track, FALSE);
+
+    path = get_file_name_verified (track);
+
     if (path)
     {
 	switch (determine_file_type (path))
@@ -2426,20 +2690,30 @@ gboolean get_gain (Track *track)
 	case FILE_TYPE_M4P: /* FIXME */
 	case FILE_TYPE_M4B: /* FIXME */
 	case FILE_TYPE_WAV: /* FIXME */
-	case FILE_TYPE_UNKNOWN: 
+	case FILE_TYPE_M4V:
+	case FILE_TYPE_MP4:
+	case FILE_TYPE_MOV:
+	case FILE_TYPE_MPG:
+	case FILE_TYPE_UNKNOWN:
+	    buf = get_track_info (track, FALSE);
 	    gtkpod_warning (
-		_("Normalization failed: file type not supported.\n"));
+		_("Normalization failed: file type not supported (%s).\n\n"),
+		buf);
+	    g_free (buf);
 	    break;
 	case FILE_TYPE_M3U: 
 	case FILE_TYPE_PLS: 
+	case FILE_TYPE_IMAGE: 
 	    break;
 	}
 	g_free (path);
     }
     else
     {
+	buf = get_track_info (track, FALSE);
 	gtkpod_warning (
-	    _("Normalization failed: file not available.\n"));
+	    _("Normalization failed: file not available (%s).\n\n"),
+	    buf);
     }
     return result;
 }
