@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-12-04 23:45:22 jcs>
+/* Time-stamp: <2006-02-25 22:56:41 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: file_itunesdb.c,v 1.72 2005/12/04 16:00:39 jcsjcs Exp $
+|  $Id: file_itunesdb.c,v 1.75 2006/02/25 14:01:43 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -927,6 +927,8 @@ static gboolean write_extended_info (iTunesDB *itdb)
 	  fprintf (fp, "thumbnail_utf8=%s\n", etr->thumb_path_utf8);
       /* this is just for convenience for people looking for a track
 	 on the ipod away from gktpod/itunes etc. */
+      if (strlen (track->ipod_path) != 0)
+	  fprintf (fp, "filename_ipod=%s\n", track->ipod_path);
       if (etr->md5_hash && *etr->md5_hash)
 	  fprintf (fp, "md5_hash=%s\n", etr->md5_hash);
       if (etr->charset && *etr->charset)
@@ -1569,8 +1571,9 @@ gboolean gp_write_itdb (iTunesDB *itdb)
   if (success)
   {
       eitdb->data_changed = FALSE;
-      /* block menu item and button */
-      display_disable_gtkpod_import_buttons();
+      /* block menu item and button if successfully written to iPod */
+      if (itdb->usertype & GP_ITDB_TYPE_IPOD)
+	  display_disable_gtkpod_import_buttons();
       gtkpod_statusbar_message(_("iPod Database Saved"));
   }
 
@@ -1698,7 +1701,29 @@ void gp_itdb_set_mountpoint (const gchar *mp)
     g_return_if_fail (gtkpod_window);
     itdbs_head = g_object_get_data (G_OBJECT (gtkpod_window),
 				    "itdbs_head");
-    if (!itdbs_head) return;
+    if (!itdbs_head)
+    {  /* this means we are called before all the data is set up, for
+	* example because gtkpod was called with the '-m' flag.
+	* We'll set all itdb_n_mountpoint for which itdb_n_type is
+	* GP_ITDB_TYPE_IPOD */
+	gboolean valid;
+	i=0;
+	do
+	{
+	    gchar *property = g_strdup_printf ("itdb_%d_type", i);
+	    gint type;
+	    valid = prefs_get_int_value (property, &type);
+	    g_free (property);
+	    if (valid && (type & GP_ITDB_TYPE_IPOD))
+	    {
+		gchar *pmp = g_strdup_printf ("itdb_%d_mountpoint", i);
+		prefs_set_string_value (pmp, mp);
+		g_free (pmp);
+	    }
+	    ++i;
+	} while (valid);
+	return;
+    }
 
     i=0;
     for (gl=itdbs_head->itdbs; gl; gl=gl->next)
