@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-07-08 23:04:28 jcs>
+/* Time-stamp: <2005-11-25 23:52:12 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: main.c,v 1.45 2005/07/09 06:53:26 jcsjcs Exp $
+|  $Id: main.c,v 1.47 2005/11/26 13:44:41 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -39,7 +39,7 @@
 #include "prefs.h"
 #include "misc.h"
 #include "file.h"
-
+//#include "podcast.h"
 
 /* path to gtkpod.glade */
 gchar *xml_file;
@@ -120,7 +120,7 @@ main (int argc, char *argv[])
 
   glade_xml_signal_autoconnect (main_window_xml);
   
-  gtkpod_window = glade_xml_get_widget (main_window_xml, "gtkpod");
+  gtkpod_window = gtkpod_xml_get_widget (main_window_xml, "gtkpod");
   
   if (!read_prefs (gtkpod_window, argc, argv)) return 0;
 
