@@ -1,7 +1,10 @@
-/*
+/* Time-stamp: <2005-06-17 22:12:15 jcs>
+|
 |  Copyright (C) 2002 Corey Donohoe <atmos at atmos.org>
+|  Copyright (C) 2004-2005 Jorg Schuler <jcsjcs at users.sourceforge.net>
 |  Part of the gtkpod project.
 | 
+|  URL: http://www.gtkpod.org/
 |  URL: http://gtkpod.sourceforge.net/
 | 
 |  This program is free software; you can redistribute it and/or modify
@@ -16,28 +19,28 @@
 | 
 |  You should have received a copy of the GNU General Public License
 |  along with this program; if not, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+|  USA
 | 
 |  iTunes and iPod are trademarks of Apple
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: md5.h,v 1.16 2004/03/23 15:54:29 jcsjcs Exp $
+|  $Id: md5.h,v 1.20 2005/06/17 13:30:03 jcsjcs Exp $
 */
 #ifndef _GTKPOD_MD5_H_
 #define _GTKPOD_MD5_H_
 
-#include "track.h"
-#include <glib.h>
-
+#include "display_itdb.h"
 
 gchar *md5_hash_on_filename (gchar *name, gboolean silent);
 /* Any calls to the following functions immediately return if md5sums
  * is not on */
-Track *md5_file_exists (gchar *file, gboolean silent);
-Track *md5_md5_exists (gchar *md5);
-Track *md5_track_exists(Track *s);
-Track *md5_track_exists_insert(Track *s);
-void md5_track_removed(Track *s);
-void md5_unique_file_free(void);
+Track *md5_file_exists (iTunesDB *itdb, gchar *file, gboolean silent);
+Track *md5_md5_exists (iTunesDB *itdb, gchar *md5);
+Track *md5_track_exists (iTunesDB *itdb, Track *s);
+Track *md5_track_exists_insert (iTunesDB *itdb, Track *s);
+void md5_track_remove (Track *s);
+void md5_free (iTunesDB *itdb);
+void md5_free_eitdb (ExtraiTunesDBData *eitdb);
 #endif
