@@ -1,5 +1,5 @@
 /* -*- coding: utf-8; -*-
-|  Time-stamp: <2005-07-15 23:37:16 jcs>
+|  Time-stamp: <2005-12-11 00:15:39 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -25,7 +25,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc.c,v 1.181 2005/07/15 14:39:37 jcsjcs Exp $
+|  $Id: misc.c,v 1.192 2005/12/10 17:04:28 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -71,16 +71,16 @@ void open_about_window ()
 
   about_xml = glade_xml_new (xml_file, "gtkpod_about_window", NULL);
   glade_xml_signal_autoconnect (about_xml);
-  about_window = glade_xml_get_widget (about_xml, "gtkpod_about_window");
+  about_window = gtkpod_xml_get_widget (about_xml, "gtkpod_about_window");
   
   
-  about_label = GTK_LABEL (glade_xml_get_widget (about_xml, "about_label"));
+  about_label = GTK_LABEL (gtkpod_xml_get_widget (about_xml, "about_label"));
   label_text = g_strdup_printf (_("gtkpod Version %s: Cross-Platform Multi-Lingual Interface to Apple's iPod(tm)."), VERSION);
   gtk_label_set_text (about_label, label_text);
   g_free (label_text);
   {
       gchar *text[] = {_("\
-(C) 2002 - 2003\n\
+(C) 2002 - 2005\n\
 Jorg Schuler (jcsjcs at users dot sourceforge dot net)\n\
 Corey Donohoe (atmos at atmos dot org)\n\
 \n\
@@ -151,7 +151,21 @@ Daniel Kercher: sync scripts for abook and webcalendar\n"),
 Clinton Gormley: sync scripts for thunderbird\n"),
 		       "\n",
 		       _("\
-Icons of buttons were made by Nicolas Chariot.\n\
+Sebastien Beridot: sync script for ldif addressbook format\n"),
+		       "\n",
+		       _("\
+Sebastian Scherer: sync script for kNotes\n"),
+		       "\n",
+		       _("\
+Nick Piper: sync script for Palm, type-ahead search\n"),
+		       "\n",
+		       _("\
+Uwe Hermann: help with support for iPod Video\n"),
+		       "\n",
+		       _("\
+Iain Benson: support for compilation tag in mp3 files and separate display of compilations in the sort tab.\n"),
+		       _("\
+Nicolas Chariot: icons of buttons\n\
 \n\
 \n"),
 		       _("\
@@ -169,7 +183,7 @@ This program borrows code from the following projects:\n"),
 The GUI was created with the help of glade-2 (http://glade.gnome.org/).\n"),
 		       NULL };
       gchar **strp = text;
-      textview = GTK_TEXT_VIEW (glade_xml_get_widget (about_xml, "credits_textview"));
+      textview = GTK_TEXT_VIEW (gtkpod_xml_get_widget (about_xml, "credits_textview"));
       tb = gtk_text_view_get_buffer (textview);
       while (*strp)
       {
@@ -196,7 +210,7 @@ Japanese: Kentaro Fukuchi (fukuchi at users dot sourceforge dot net)\n"),
 Swedish: Stefan Asserhall (stefan asserhall at comhem dot se)\n"),
 				     NULL };
       gchar **strp = text;
-      textview = GTK_TEXT_VIEW (glade_xml_get_widget (about_xml, "translators_textview"));
+      textview = GTK_TEXT_VIEW (gtkpod_xml_get_widget (about_xml, "translators_textview"));
       tb = gtk_text_view_get_buffer (textview);
       while (*strp)
       {
@@ -322,16 +336,14 @@ void cleanup_backup_and_extended_files (void)
 }
 
 
-/* used by glist_duplicate */
-static void gl_dup_fe (gpointer data, GList **dup)
-{
-    *dup = g_list_append (*dup, data);
-}
-
-
-/* Duplicate a GList */
+/* Duplicate a GList (shallow copy) */
 GList *glist_duplicate (GList *list)
 {
+    auto void gl_dup_fe (gpointer data, GList **dup);
+    void gl_dup_fe (gpointer data, GList **dup)
+	{
+	    *dup = g_list_append (*dup, data);
+	}
     GList *dup = NULL;
     g_list_foreach (list, (GFunc)gl_dup_fe, &dup);
     return dup;
@@ -347,6 +359,8 @@ GList *glist_duplicate (GList *list)
  * does not check prefs to see if the current prefs want gtkpod itself to
  * mount the ipod drive, that should be checked before making this call.
  */
+#include <sys/param.h>   /* seems to be needed for FreeBSD 5.4
+			    otherwise sys/mount.h throws an error */
 #include <sys/mount.h>
 #include <errno.h>
 #include <stdio.h>
@@ -545,7 +559,7 @@ void call_script (gchar *script)
 
 /* compare @str1 and @str2 case-sensitively or case-insensitively
  * depending on prefs settings */
-gint compare_string (gchar *str1, gchar *str2)
+gint compare_string (const gchar *str1, const gchar *str2)
 {
     if (prefs_get_case_sensitive ())
 	return strcmp (str1, str2);
@@ -613,7 +627,7 @@ void compare_string_fuzzy_generate_keys (void)
 /* compare @str1 and @str2 case-sensitively or case-insensitively
  * depending on prefs settings, and ignoring certain initial articles 
  * ("the", "le"/"la", etc) */
-gint compare_string_fuzzy (gchar *str1, gchar *str2)
+gint compare_string_fuzzy (const gchar *str1, const gchar *str2)
 {
     gchar *tempStr;
     gint   result;
@@ -621,8 +635,8 @@ gint compare_string_fuzzy (gchar *str1, gchar *str2)
     gchar *cleanStr1 = g_utf8_casefold (str1, -1);
     gchar *cleanStr2 = g_utf8_casefold (str2, -1);
 
-    gchar *pstr1 = str1;
-    gchar *pstr2 = str2;
+    const gchar *pstr1 = str1;
+    const gchar *pstr2 = str2;
     gchar *pcleanStr1 = cleanStr1;
     gchar *pcleanStr2 = cleanStr2;
     
@@ -685,7 +699,7 @@ gint compare_string_fuzzy (gchar *str1, gchar *str2)
 
 /* compare @str1 and @str2 case-sensitively or case-insensitively
  * depending on prefs settings */
-gint compare_string_case_insensitive (gchar *str1, gchar *str2)
+gint compare_string_case_insensitive (const gchar *str1, const gchar *str2)
 {
     gchar *string1 = g_utf8_casefold (str1, -1);
     gchar *string2 = g_utf8_casefold (str2, -1);
@@ -695,6 +709,30 @@ gint compare_string_case_insensitive (gchar *str1, gchar *str2)
     return result;
 }
 
+/* todo: optionally ignore 'the', 'a,' etc. */
+gboolean compare_string_start_case_insensitive (const gchar *haystack, const gchar *needle)
+{
+  gint cmp = 0;
+  gchar *nhaystack = g_utf8_normalize(haystack, -1, G_NORMALIZE_ALL);
+  gchar *lhaystack = g_utf8_casefold(nhaystack,-1);
+  gchar *nneedle = g_utf8_normalize(needle, -1, G_NORMALIZE_ALL);
+  gchar *lneedle = g_utf8_casefold(nneedle,-1);
+
+
+  cmp = strncmp(lhaystack, lneedle, strlen(lneedle));
+
+  /*
+    printf("searched for %s , matching against %s with %d bytes. say=%d\n", 
+    lneedle, lhaystack, strlen(lneedle), cmp);
+  */
+  
+  g_free(nhaystack);
+  g_free(lhaystack);
+  g_free(nneedle);
+  g_free(lneedle);
+  return cmp;
+};
+
 
 /* ------------------------------------------------------------
 ------------------------------------------------------------------
@@ -769,7 +807,7 @@ void option_set_radio_button (GladeXML *win_xml,
 	prefs_set_int_value (prefs_string, 0);
 	wnum = 0;
     }
-    w = glade_xml_get_widget (win_xml, widgets[wnum]);
+    w = gtkpod_xml_get_widget (win_xml, widgets[wnum]);
     if (w)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), TRUE);
 }
@@ -787,7 +825,7 @@ gint option_get_radio_button (GladeXML *win_xml,
 
     for (i=0; widgets[i]; ++i)
     {
-	GtkWidget *w = glade_xml_get_widget (win_xml, widgets[i]);
+	GtkWidget *w = gtkpod_xml_get_widget (win_xml, widgets[i]);
 	if (w)
 	{
 	    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)))
@@ -889,7 +927,7 @@ void option_set_string (GladeXML *win_xml,
     if (!string)
 	string = g_strdup (dflt);
 
-    entry = glade_xml_get_widget (win_xml, name);
+    entry = gtkpod_xml_get_widget (win_xml, name);
 
     if (entry)
 	gtk_entry_set_text(GTK_ENTRY(entry), string);
@@ -909,7 +947,7 @@ void option_get_string (GladeXML *win_xml,
 
     g_return_if_fail (win_xml && name);
 
-    entry = glade_xml_get_widget (win_xml, name);
+    entry = gtkpod_xml_get_widget (win_xml, name);
 
     if (entry)
     {
@@ -934,7 +972,7 @@ void option_set_toggle_button (GladeXML *win_xml,
     if (!prefs_get_int_value (name, &active))
 	active = dflt;
 
-    button = glade_xml_get_widget (win_xml, name);
+    button = gtkpod_xml_get_widget (win_xml, name);
 
     if (button)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button),
@@ -952,7 +990,7 @@ gboolean option_get_toggle_button (GladeXML *win_xml,
 
     g_return_val_if_fail (win_xml && name, active);
 
-    button = glade_xml_get_widget (win_xml, name);
+    button = gtkpod_xml_get_widget (win_xml, name);
 
     if (button)
     {
@@ -1019,7 +1057,10 @@ static gchar *fix_path(gchar *orig)
 /* Match a list of templates @p separated by ';' with the type of the
    filename. E.g. '%s.mp3;%t.wav' will return '%s.mp3' if @track is an
    mp3 file, or '%t.wav' if @track is a wav file. If no template can
-   be matched, an empty string is returned. */
+   be matched, an empty string is returned.
+
+   String be freed after use.
+*/
 static gchar *select_template (Track *track, const gchar *p)
 {
     gchar **templates, **tplp;
@@ -1067,56 +1108,38 @@ static gchar *select_template (Track *track, const gchar *p)
 }
 
 
-/* Return a string for @track built according to @full_template.
-   @full_template can contain several templates separated by ';',
-   e.g. '%s.mp3;%t.wav'. The correct one is selected using
-   select_template() defined above.
-   If @is_filename is TRUE, potentially harmful characters are
-   replaced in an attempt to create a valid filename.
-   If @is_filename is FALSE, the extension (e.g. '.mp3' will be
-   removed). */
+/* Return a string for @track built according to @template.
+
+   @is_filename: if TRUE, remove potentially harmful characters.*/
 gchar *get_string_from_template (Track *track,
-				 const gchar *full_template,
+				 const gchar *template,
 				 gboolean is_filename)
 {
     GString *result;
-    gchar *p, *res_utf8;
+    gchar *res_utf8;
+    const gchar *p;
     gchar *basename = NULL;
-    gchar *template;
+    gchar *basename_noext = NULL;
     ExtraTrackData *etr;
 
-    g_return_val_if_fail (track && full_template, NULL);
+    g_return_val_if_fail (track, NULL);
+    g_return_val_if_fail (template, NULL);
     etr = track->userdata;
     g_return_val_if_fail (etr, NULL);
 
-    template = select_template (track, full_template);
-
-    if (!template)
-    {
-	gchar *fn = get_file_name (track);
-	gtkpod_warning (_("Template ('%s') does not match file type '%s'\n"), full_template, fn ? fn:"");
-	g_free (fn);
-	return NULL;
-    }
-
-    if (!is_filename)
-    {   /* remove an extension, if present ('.???' or '.????'  at the
-	   end) */
-	gchar *pnt = strrchr (template, '.');
-	if (pnt)
-	{
-	    if (pnt == template+strlen(template)-3)
-		*pnt = 0;
-	    if (pnt == template+strlen(template)-4)
-		*pnt = 0;
-	}
-    }
-
     result = g_string_new ("");
 
     /* try to get the original filename */
     if (etr->pc_path_utf8)
 	basename = g_path_get_basename (etr->pc_path_utf8);
+    /* get original filename without extension */
+    if (basename)
+    {
+	gchar *ptr;
+	basename_noext = g_strdup (basename);
+	ptr = strrchr (basename_noext, '.');
+	if (ptr) *ptr = '\0';
+    }
 
     p=template;
     while (*p != '\0') {
@@ -1132,6 +1155,12 @@ gchar *get_string_from_template (Track *track,
 		    tmp = basename;
 		}
 		break;
+	    case 'O':
+		if (basename_noext)
+		{
+		    tmp = basename_noext;
+		}
+		break;
 	    case 'p':
 		pl = pm_get_selected_playlist ();
 		if (pl)
@@ -1236,6 +1265,61 @@ gchar *get_string_from_template (Track *track,
 	}
     }
 
+    g_free (basename);
+    g_free (basename_noext);
+
+    return res_utf8;
+}
+
+
+
+/* Return a string for @track built according to @full_template.
+   @full_template can contain several templates separated by ';',
+   e.g. '%s.mp3;%t.wav'. The correct one is selected using
+   select_template() defined above.
+
+   If @is_filename is TRUE, potentially harmful characters are
+   replaced in an attempt to create a valid filename.
+
+   If @is_filename is FALSE, the extension (e.g. '.mp3' will be
+   removed). */
+gchar *get_string_from_full_template (Track *track,
+				      const gchar *full_template,
+				      gboolean is_filename)
+{
+    gchar *res_utf8;
+    gchar *template;
+
+    g_return_val_if_fail (track, NULL);
+    g_return_val_if_fail (full_template, NULL);
+
+    template = select_template (track, full_template);
+
+    if (!template)
+    {
+	gchar *fn = get_file_name (track);
+	gtkpod_warning (_("Template ('%s') does not match file type '%s'\n"), full_template, fn ? fn:"");
+	g_free (fn);
+	return NULL;
+    }
+
+    if (!is_filename)
+    {   /* remove an extension, if present ('.???' or '.????'  at the
+	   end) */
+	gchar *pnt = strrchr (template, '.');
+	if (pnt)
+	{
+	    if (pnt == template+strlen(template)-3)
+		*pnt = 0;
+	    if (pnt == template+strlen(template)-4)
+		*pnt = 0;
+	}
+    }
+
+    res_utf8 = get_string_from_template (track, template, is_filename);
+
+    g_free (template);
+
     return res_utf8;
 }
 
@@ -1267,3 +1351,87 @@ gchar *which (const gchar *exe)
     g_free(which_exec);
     return(result);
 }
+
+/**
+ * Recursively make directories.
+ * @return FALSE is this is not possible.
+ */
+gboolean mkdirhier(const gchar *dirname)
+{
+    gchar *dn, *p;
+
+    g_return_val_if_fail (dirname && *dirname, FALSE);
+
+    if (strncmp ("~/", dirname, 2) == 0)
+	 dn = g_build_filename (g_get_home_dir(), dirname+2, NULL);
+    else dn = g_strdup (dirname);
+
+    p = dn;
+
+    do
+    {
+	++p;
+	p = index (p, G_DIR_SEPARATOR);
+
+	if (p)   *p = '\0';
+
+	if (!g_file_test(dn, G_FILE_TEST_EXISTS))
+	{
+	    if (mkdir(dn, 0777) == -1)
+	    {
+		gtkpod_warning (_("Error creating %s: %s\n"),
+				dn, g_strerror(errno));
+		g_free (dn);
+		return FALSE;
+	    }
+	}
+	if (p)   *p = G_DIR_SEPARATOR;
+    } while (p);
+
+    g_free (dn);
+    return TRUE;
+}
+
+/**
+ * Recursively make directories in the given filename.
+ * @return FALSE is this is not possible.
+ */
+gboolean mkdirhierfile(const gchar *filename)
+{
+    gboolean result;
+    gchar *dirname = g_path_get_dirname (filename);
+    result = mkdirhier (dirname);
+    g_free (dirname);
+    return result;
+}
+
+
+/**
+ * Convert "~/" to "/home/.../"
+ *
+ * g_free() return value when no longer needed.
+ */
+gchar *convert_filename (const gchar *filename)
+{
+    if (strncmp ("~/", filename, 2) == 0)
+	 return g_build_filename (g_get_home_dir(), filename+2, NULL);
+    else return g_strdup (filename);
+}
+
+
+
+/**
+ * Wrapper for gtkpod_xml_get_widget() giving out a warning if widget
+ * could not be found.
+ *
+ **/
+GtkWidget *gtkpod_xml_get_widget (GladeXML *xml, const gchar *name)
+{
+    GtkWidget *w=glade_xml_get_widget (xml, name);
+
+    if (!w)
+	fprintf (stderr, "*** Programming error: Widget not found: '%s'\n",
+		 name);
+
+    return w;
+}
