@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-07-16 01:51:01 jcs>
+/* Time-stamp: <2005-12-09 23:52:06 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc_confirm.c,v 1.19 2005/07/15 16:58:42 jcsjcs Exp $
+|  $Id: misc_confirm.c,v 1.25 2005/12/09 14:55:34 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -536,11 +536,18 @@ static void delete_playlist_ok (struct DeleteData *dd)
 		gp_playlist_remove_track (dd->pl, dd->pl->members->data,
 					  dd->deleteaction);
 	    }
-	    if (dd->pl->type == ITDB_PL_TYPE_MPL)
+	    if (itdb_playlist_is_mpl (dd->pl))
 	    {
 		msg = g_strdup_printf (_("Removed all %d tracks from the iPod"), n);
 		display_reset (0);
 	    }
+	    else if (itdb_playlist_is_podcasts (dd->pl))
+	    {
+		msg = g_strdup_printf (_("Removed all podcasts from the iPod"));
+		if (pm_get_selected_playlist () == dd->pl)
+		    st_redisplay (0);
+//		display_reset (0);
+	    }
 	    else
 	    {
 		/* remove playlist */
@@ -553,7 +560,7 @@ static void delete_playlist_ok (struct DeleteData *dd)
 	    }
 	    break;
 	case DELETE_ACTION_PLAYLIST:
-	    if (dd->pl->type == ITDB_PL_TYPE_MPL)
+	    if (itdb_playlist_is_mpl (dd->pl))
 	    {	/* not allowed -- programming error */
 		g_return_if_reached ();
 	    }
@@ -576,7 +583,7 @@ static void delete_playlist_ok (struct DeleteData *dd)
 	switch (dd->deleteaction)
 	{
 	case DELETE_ACTION_LOCAL:
-	    if (dd->pl->type == ITDB_PL_TYPE_MPL)
+	    if (itdb_playlist_is_mpl (dd->pl))
 	    {   /* for safety reasons this is not implemented (would
 		   remove all tracks from your local harddisk */
 		g_return_if_reached ();
@@ -606,7 +613,7 @@ static void delete_playlist_ok (struct DeleteData *dd)
 		gp_playlist_remove_track (dd->pl, dd->pl->members->data,
 					  dd->deleteaction);
 	    }
-	    if (dd->pl->type == ITDB_PL_TYPE_MPL)
+	    if (itdb_playlist_is_mpl (dd->pl))
 	    {
 		msg = g_strdup_printf (_("Removed all %d tracks from the database"), n);
 		display_reset (0);
@@ -623,7 +630,7 @@ static void delete_playlist_ok (struct DeleteData *dd)
 	    }
 	    break;
 	case DELETE_ACTION_PLAYLIST:
-	    if (dd->pl->type == ITDB_PL_TYPE_MPL)
+	    if (itdb_playlist_is_mpl (dd->pl))
 	    {	/* not allowed -- programming error */
 		g_return_if_reached ();
 	    }
@@ -681,10 +688,15 @@ void delete_playlist_head (DeleteAction deleteaction)
 	switch (deleteaction)
 	{
 	case DELETE_ACTION_IPOD:
-	    if (pl->type == ITDB_PL_TYPE_MPL)
+	    if (itdb_playlist_is_mpl (pl))
 	    {
 		label = g_strdup_printf (_("Are you sure you want to remove all tracks from your iPod?"));
 	    }
+	    else if (itdb_playlist_is_podcasts (pl))
+	    {   /* podcasts playlist */
+		dd->selected_tracks = g_list_copy (pl->members);
+		label = g_strdup_printf (_("Are you sure you want to remove all podcasts from your iPod?"));
+	    }
 	    else 
 	    {   /* normal playlist */
 		/* we set selected_tracks to get a list printed by
@@ -696,7 +708,7 @@ void delete_playlist_head (DeleteAction deleteaction)
 	    }
 	    break;
 	case DELETE_ACTION_PLAYLIST:
-	    if (pl->type == ITDB_PL_TYPE_MPL)
+	    if (itdb_playlist_is_mpl (pl))
 	    {	/* not allowed -- programming error */
 		g_return_if_reached ();
 	    }
@@ -717,7 +729,7 @@ void delete_playlist_head (DeleteAction deleteaction)
 	switch (deleteaction)
 	{
 	case DELETE_ACTION_LOCAL:
-	    if (pl->type == ITDB_PL_TYPE_MPL)
+	    if (itdb_playlist_is_mpl (pl))
 	    {   /* for safety reasons this is not implemented (would
 		   remove all tracks from your local harddisk */
 		g_return_if_reached ();
@@ -734,7 +746,7 @@ void delete_playlist_head (DeleteAction deleteaction)
 	    }
 	    break;
 	case DELETE_ACTION_DATABASE:
-	    if (pl->type == ITDB_PL_TYPE_MPL)
+	    if (itdb_playlist_is_mpl (pl))
 	    {
 		label = g_strdup_printf (_("Are you sure you want to remove all tracks from the database?"));
 		
@@ -751,7 +763,7 @@ void delete_playlist_head (DeleteAction deleteaction)
 	    }
 	    break;
 	case DELETE_ACTION_PLAYLIST:
-	    if (pl->type == ITDB_PL_TYPE_MPL)
+	    if (itdb_playlist_is_mpl (pl))
 	    {	/* not allowed -- programming error */
 		g_return_if_reached ();
 	    }
@@ -864,55 +876,66 @@ static void ipod_directories_ok (gchar *mp)
 	pbuf = g_build_filename (mp, "Calendars", NULL);
 	if (!g_file_test (pbuf, G_FILE_TEST_EXISTS))
 	{
-	    if((mkdir(pbuf, 0755) != 0))
+	    if((mkdir(pbuf, 0777) != 0))
 		errordir = pbuf;
-	    else
-		g_free (pbuf);
 	}
+	if (!errordir)
+	    g_free (pbuf);
     }
     if (!errordir)
     {
 	pbuf = g_build_filename (mp, "Contacts", NULL);
 	if (!g_file_test (pbuf, G_FILE_TEST_EXISTS))
 	{
-	    if((mkdir(pbuf, 0755) != 0))
+	    if((mkdir(pbuf, 0777) != 0))
 		errordir = pbuf;
-	    else
-		g_free (pbuf);
 	}
+	if (!errordir)
+	    g_free (pbuf);
     }
     if (!errordir)
     {
 	pbuf = g_build_filename (mp, "iPod_Control", NULL);
 	if (!g_file_test (pbuf, G_FILE_TEST_EXISTS))
 	{
-	    if((mkdir(pbuf, 0755) != 0))
+	    if((mkdir(pbuf, 0777) != 0))
 		errordir = pbuf;
-	    else
-		g_free (pbuf);
 	}
+	if (!errordir)
+	    g_free (pbuf);
     }
     if (!errordir)
     {
 	pbuf = g_build_filename (mp, "iPod_Control", "Music", NULL);
 	if (!g_file_test (pbuf, G_FILE_TEST_EXISTS))
 	{
-	    if((mkdir(pbuf, 0755) != 0))
+	    if((mkdir(pbuf, 0777) != 0))
 		errordir = pbuf;
-	    else
-		g_free (pbuf);
 	}
+	if (!errordir)
+	    g_free (pbuf);
     }
     if (!errordir)
     {
 	pbuf = g_build_filename (mp, "iPod_Control", "iTunes", NULL);
 	if (!g_file_test (pbuf, G_FILE_TEST_EXISTS))
 	{
-	    if((mkdir(pbuf, 0755) != 0))
+	    if((mkdir(pbuf, 0777) != 0))
 		errordir = pbuf;
-	    else
-		g_free (pbuf);
 	}
+	if (!errordir)
+	    g_free (pbuf);
+    }
+    if (!errordir)
+    {
+	pbuf = g_build_filename (mp, "iPod_Control", "Artwork", NULL);
+	if (!g_file_test (pbuf, G_FILE_TEST_EXISTS))
+	{
+	    if((mkdir(pbuf, 0777) != 0))
+		errordir = pbuf;
+	}
+	if (!errordir)
+	    g_free (pbuf);
     }
     dirnum = ipod_directories_number (mp);
     for(i = 0; i < dirnum; i++)
@@ -924,12 +947,12 @@ static void ipod_directories_ok (gchar *mp)
 				     "iPod_Control", "Music", num, NULL);
 	    if (!g_file_test (pbuf, G_FILE_TEST_EXISTS))
 	    {
-		if((mkdir(pbuf, 0755) != 0))
+		if((mkdir(pbuf, 0777) != 0))
 		    errordir = pbuf;
-		else
-		    g_free (pbuf);
 		g_free (num);
 	    }
+	    if (!errordir)
+		g_free (pbuf);
 	}
     }
 
