@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-07-02 00:37:36 jcs>
+/* Time-stamp: <2005-12-03 20:39:08 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc_conversion.c,v 1.19 2005/07/01 16:53:44 jcsjcs Exp $
+|  $Id: misc_conversion.c,v 1.25 2005/12/03 16:36:26 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -64,7 +64,7 @@ static const gchar *t_strings[] = {
     N_("Genre"),
     N_("Comment"),           /*  5 */
     N_("Composer"),
-    N_("Kind"),
+    N_("File type"),
     N_("PC File"),
     N_("iPod File"),
     N_("iPod ID"),           /* 10 */
@@ -86,6 +86,13 @@ static const gchar *t_strings[] = {
     N_("CD Nr"),
     N_("Grouping"),
     N_("Compilation"),
+    N_("Category"),
+    N_("Description"),       /* 30 */
+    N_("Podcast URL"),
+    N_("Podcast RSS"),
+    N_("Subtitle"),
+    N_("Date released"),
+    N_("Checked"),           /* 35 */
     NULL };
 
 /* Tooltips for prefs window */
@@ -121,7 +128,16 @@ static const gchar *t_tooltips[] = {
     NULL,                                              /* 25 */
     N_("CD Nr. and total number of CDS in set"),
     NULL,
-    NULL };
+    NULL,
+    N_("The category (e.g. 'Technology' or 'Music') where the podcast was located."),
+    N_("Accessible by selecting the center button on the iPod."), /* 30 */
+    NULL,
+    NULL,
+    NULL,
+    N_("Release date (for podcasts displayed next to the title on the iPod)"),
+    NULL,   /* 35 */
+    NULL
+ };
 
 
 /* translates a TM_COLUMN_... (defined in display.h) into a
@@ -136,7 +152,7 @@ T_item TM_to_T (TM_item tm)
     case TM_COLUMN_ALBUM:         return T_ALBUM;
     case TM_COLUMN_GENRE:         return T_GENRE;
     case TM_COLUMN_COMPOSER:      return T_COMPOSER;
-    case TM_COLUMN_FDESC:         return T_FDESC;
+    case TM_COLUMN_FILETYPE:      return T_FILETYPE;
     case TM_COLUMN_GROUPING:      return T_GROUPING;
     case TM_COLUMN_TRACK_NR:      return T_TRACK_NR;
     case TM_COLUMN_CD_NR:         return T_CD_NR;
@@ -158,6 +174,13 @@ T_item TM_to_T (TM_item tm)
     case TM_COLUMN_SOUNDCHECK:    return T_SOUNDCHECK;
     case TM_COLUMN_YEAR:          return T_YEAR;
     case TM_COLUMN_COMPILATION:   return T_COMPILATION;
+    case TM_COLUMN_COMMENT:       return T_COMMENT;
+    case TM_COLUMN_CATEGORY:      return T_CATEGORY;
+    case TM_COLUMN_DESCRIPTION:   return T_DESCRIPTION;
+    case TM_COLUMN_PODCASTURL:    return T_PODCASTURL;
+    case TM_COLUMN_PODCASTRSS:    return T_PODCASTRSS;
+    case TM_COLUMN_SUBTITLE:      return T_SUBTITLE;
+    case TM_COLUMN_TIME_RELEASED: return T_TIME_RELEASED;
     case TM_NUM_COLUMNS:          return -1;
     }
     return -1;
@@ -191,8 +214,9 @@ const gchar *get_tm_string (TM_item tm)
 {
     T_item t = TM_to_T (tm);
 
-    if (t != -1)   return t_strings[t];
-    else           return ("");
+    g_return_val_if_fail (t != -1, "");
+
+    return t_strings[t];
 }
 
 
@@ -203,8 +227,9 @@ const gchar *get_tm_tooltip (TM_item tm)
 {
     T_item t = TM_to_T (tm);
 
-    if (t != -1)   return t_tooltips[t];
-    else           return ("");
+    g_return_val_if_fail (t != -1, "");
+
+    return t_tooltips[t];
 }
 
 
@@ -213,8 +238,9 @@ const gchar *get_tm_tooltip (TM_item tm)
  * buttons or column headers). */
 const gchar *get_t_string (T_item t)
 {
-    if (t != -1)   return t_strings[t];
-    else           return ("");
+    g_return_val_if_fail (t>=0 && t<T_ITEM_NUM, "");
+
+    return t_strings[t];
 }
 
 
@@ -317,7 +343,7 @@ time_t time_string_to_time (const gchar *str)
 
 
 /* convert the string @str to a time stamp, assuming 0:00:00 if no
- * time is given */
+ * time is given. Returns 0 if @str is "--" */
 time_t time_string_to_fromtime (const gchar *str)
 {
     time_t t;
@@ -325,6 +351,8 @@ time_t time_string_to_fromtime (const gchar *str)
 
     g_return_val_if_fail (str, -1);
 
+    if (strcmp (str, "--") == 0)  return 0;
+
     t = time (NULL);
     localtime_r (&t, &tm);
     tm.tm_sec = 0;
@@ -356,22 +384,25 @@ time_t time_string_to_totime (const gchar *str)
 }
 
 
-/* get the timestamp TM_COLUMN_TIME_CREATE/PLAYED/MODIFIED */
-time_t time_get_time (Track *track, TM_item tm_item)
+/* get the timestamp TM_COLUMN_TIME_CREATE/PLAYED/MODIFIED/RELEASED */
+time_t time_get_time (Track *track, T_item t_item)
 {
     guint32 mactime = 0;
 
-    if (track) switch (tm_item)
+    if (track) switch (t_item)
     {
-    case TM_COLUMN_TIME_ADDED:
+    case T_TIME_ADDED:
 	mactime = track->time_added;
 	break;
-    case TM_COLUMN_TIME_PLAYED:
+    case T_TIME_PLAYED:
 	mactime = track->time_played;
 	break;
-    case TM_COLUMN_TIME_MODIFIED:
+    case T_TIME_MODIFIED:
 	mactime = track->time_modified;
 	break;
+    case T_TIME_RELEASED:
+	mactime = track->time_released;
+	break;
     default:
 	mactime = 0;
 	break;
@@ -381,28 +412,31 @@ time_t time_get_time (Track *track, TM_item tm_item)
 
 
 /* hopefully obvious */
-gchar *time_field_to_string (Track *track, TM_item tm_item)
+gchar *time_field_to_string (Track *track, T_item t_item)
 {
-    return (time_time_to_string (time_get_time (track, tm_item)));
+    return (time_time_to_string (time_get_time (track, t_item)));
 }
 
 
-/* get the timestamp TM_COLUMN_TIME_CREATE/PLAYED/MODIFIED */
-void time_set_time (Track *track, time_t time, TM_item tm_item)
+/* get the timestamp TM_COLUMN_TIME_CREATE/PLAYED/MODIFIED/RELEASED */
+void time_set_time (Track *track, time_t time, T_item t_item)
 {
     guint32 mactime = itdb_time_host_to_mac (time);
 
-    if (track) switch (tm_item)
+    if (track) switch (t_item)
     {
-    case TM_COLUMN_TIME_ADDED:
+    case T_TIME_ADDED:
 	track->time_added = mactime;
 	break;
-    case TM_COLUMN_TIME_PLAYED:
+    case T_TIME_PLAYED:
 	track->time_played = mactime;
 	break;
-    case TM_COLUMN_TIME_MODIFIED:
+    case T_TIME_MODIFIED:
 	track->time_modified = mactime;
 	break;
+    case T_TIME_RELEASED:
+	track->time_released = mactime;
+	break;
     default:
 	break;
     }
