@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-07-16 15:42:28 jcs>
+/* Time-stamp: <2005-11-21 00:27:51 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc_playlist.c,v 1.23 2005/07/16 07:13:50 jcsjcs Exp $
+|  $Id: misc_playlist.c,v 1.27 2005/11/23 04:16:12 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -173,7 +173,7 @@ void generate_category_playlists (iTunesDB *itdb, T_item cat)
 	Track *track = gl->data;
 	Playlist *cat_pl = NULL;
 	gchar *category = NULL;
-	gchar *track_cat;
+	const gchar *track_cat;
 
 	track_cat = track_get_item (track, cat);
 
@@ -638,7 +638,7 @@ static gint All_Ratings_CF (gconstpointer aa, gconstpointer bb)
 /* Insert function: determines whether a track is entered into the playlist */
 static gboolean All_Ratings_IF (Track *track, gpointer user_data)
 {
-    gint playlist_nr = (gint)user_data;
+    guint playlist_nr = GPOINTER_TO_UINT(user_data);
     if (track) return (track->rating == playlist_nr*20);
     return FALSE;
 }
@@ -647,7 +647,7 @@ static gboolean All_Ratings_IF (Track *track, gpointer user_data)
 void each_rating_pl(iTunesDB *itdb)
 {
     gchar *str;
-    gint playlist_nr;
+    guint playlist_nr;
 
     g_return_if_fail (itdb);
     str = _("Unrated tracks");
@@ -659,7 +659,7 @@ void each_rating_pl(iTunesDB *itdb)
 	} 
     	update_ranked_playlist (itdb, str, 0,
 				All_Ratings_IF, All_Ratings_CF,
-				(gpointer)playlist_nr);
+				GUINT_TO_POINTER(playlist_nr));
     }
     g_free (str);
 }
@@ -765,9 +765,6 @@ void since_last_pl (iTunesDB *itdb)
  *  can happen... didn't check yet
  ******************************************************************************/
 
-#define IPOD_MUSIC_DIRS 20
-#define IPOD_CONTROL_DIR "iPod_Control"
-
 
 /* compare @str1 and @str2 case-sensitively only */
 gint str_cmp (gconstpointer str1, gconstpointer str2, gpointer data)
@@ -1032,18 +1029,34 @@ void check_db (iTunesDB *itdb)
 	Track *track = gl->data;
         gint ntok=0;
 	g_return_if_fail (track);
-        /* we don't want to report non-transferred files as dandgling */
+        /* we don't want to report non-transferred files as dangling */
 	if (!track->transferred) continue; 
 	tokens = g_strsplit(track->ipod_path,":",(track->ipod_path[0]==':'?4:3));
         ntok=ntokens(tokens);
 	if (ntok>=3)
+	{
 	    pathtrack=g_strdup (tokens[ntok-1]);
+	}
 	else
-	    fprintf(stderr, "Report the bug please: shouldn't be 0 at %s:%d\n",__FILE__,__LINE__);
-        if (localdebug)
-            fprintf(stdout,"File %s\n", pathtrack);
+	{
+	    /* illegal ipod_path */
+	    /* the track has NO ipod_path, so we want the item to
+	       ultimately be deleted from DB, * however, we need to
+	       add it to tree it such a way that:
+               a) it will be unique
+               b) it won't match to any existing file on the ipod
 
-	fflush(stdout);
+               so use something invented using the pointer to the
+               track structure as a way to generate uniqueness
+            */
+           pathtrack=g_strdup_printf ("NOFILE-%p", track);
+	}
+
+        if (localdebug)
+	{
+            fprintf(stdout,"File %s\n", pathtrack);
+	    fflush(stdout);
+	}
 
 	g_tree_insert (files_known, pathtrack, track);
 	g_strfreev(tokens);
@@ -1053,10 +1066,9 @@ void check_db (iTunesDB *itdb)
     gtkpod_tracks_statusbar_update();
     process_gtk_events_blocked();
 
-    for(h=0;h<IPOD_MUSIC_DIRS;h++)
+    for(h=0; h<itdb_musicdirs_number (itdb); h++)
     {
 	/* directory name */
-	/* FIXME: should not depend on IPOD_MUSIC_DIRS */
 	gchar *ipod_dir=g_strdup_printf("F%02d",h); /* just directory name */
 	gchar *ipod_fulldir;
 	const gchar * music[] = {"iPod_Control","Music", NULL, NULL,};
