@@ -1,9 +1,10 @@
-/* Time-stamp: <2004-03-24 22:26:08 JST jcs>
+/* Time-stamp: <2005-06-17 22:12:13 jcs>
 |
-|  Copyright (C) 2002-2003 Jorg Schuler <jcsjcs at users.sourceforge.net>
+|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
-|
-|  URL:  http://gtkpod.sourceforge.net/
+| 
+|  URL: http://www.gtkpod.org/
+|  URL: http://gtkpod.sourceforge.net/
 |
 |  This program is free software; you can redistribute it and/or modify
 |  it under the terms of the GNU General Public License as published by
@@ -23,13 +24,13 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: mp3file.h,v 1.21 2004/03/24 13:27:04 jcsjcs Exp $
+|  $Id: mp3file.h,v 1.23 2005/06/17 13:30:04 jcsjcs Exp $
 */
 
 #ifndef MP3FILEH_INCLUDED
 #define MP3FILEH_INCLUDED 1
 
-#include "track.h"
+#include "itdb.h"
 
 gboolean mp3_write_file_info (gchar *filename, Track *track);
 Track *mp3_get_file_info (gchar *name);
