@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-06-17 21:01:39 jcs>
+/* Time-stamp: <2005-11-12 17:16:12 jcs>
 |
 |  Copyright (C) 2002 Corey Donohoe <atmos at atmos.org>
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
@@ -25,14 +25,13 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: prefs_window.h,v 1.65 2005/06/17 13:30:05 jcsjcs Exp $
+|  $Id: prefs_window.h,v 1.68 2005/11/12 08:54:31 jcsjcs Exp $
 */
 #ifndef _GTKPOD_PREFS_WINDOW_H
 #define _GTKPOD_PREFS_WINDOW_H
 
 #include <gtk/gtk.h>
 #include "prefs.h"
-
 extern const gchar *path_entry_names[];
 
 void prefs_window_update_default_sizes (void);
@@ -42,8 +41,9 @@ void prefs_window_show_hide_tooltips (void);
 void prefs_window_ok (void);
 void prefs_window_apply (void);
 void prefs_window_cancel (void);
-void prefs_window_create (void);
+void prefs_window_create (gint page);
 void prefs_window_delete (void);
+void prefs_window_podcasts (void);
 
 void prefs_window_set_unused_gboolean3 (gboolean val);
 
@@ -63,4 +63,7 @@ void sort_window_set_st_sort (gint val);
 void sort_window_set_tm_sort (gint val);
 void sort_window_set_case_sensitive (gboolean val);
 void sort_window_update (void);
+
+void create_subs_treeview (void);
 #endif
+
