@@ -1,4 +1,4 @@
-/* Time-stamp: <2005-06-17 22:12:12 jcs>
+/* Time-stamp: <2005-09-17 21:55:20 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: wavfile.c,v 1.8 2005/06/17 13:30:05 jcsjcs Exp $
+|  $Id: wavfile.c,v 1.9 2005/09/20 09:50:13 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -165,7 +165,7 @@ Track *wav_get_file_info (gchar *filename)
     track->samplerate = wav_file->samples_per_sec;
     track->tracklen = 1000 * ((double)8*len / track->bitrate);
     track->bitrate /= 1000; /* change to kbps */
-    track->fdesc = g_strdup ("WAV audio file");
+    track->filetype = g_strdup ("WAV audio file");
 
     fclose(wav_file->file);
     g_free(wav_file);
